"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getModulesTree = exports.invalidateModule = void 0;
/**
 * Removes module and it's deep importers from the module cache
 */
const invalidateModule = (modulePath, requireCache) => {
    for (const { filename } of getModulesTree(modulePath, requireCache)) {
        requireCache.delete(filename);
    }
};
exports.invalidateModule = invalidateModule;
/**
 * A generator which given a module path and a require cache will yield all files which deeply depend on given module
 */
function* getModulesTree(modulePath, moduleCache, visited = new Set()) {
    if (visited.has(modulePath)) {
        return;
    }
    visited.add(modulePath);
    const fileModule = moduleCache.get(modulePath);
    if (!fileModule) {
        return;
    }
    yield fileModule;
    for (const [moduleFileName, cachedModule] of moduleCache) {
        if (cachedModule.children.includes(fileModule)) {
            yield* getModulesTree(moduleFileName, moduleCache, visited);
        }
    }
}
exports.getModulesTree = getModulesTree;
//# sourceMappingURL=invalidate-modules.js.map