"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createDependencyResolver = void 0;
const { hasOwnProperty } = Object.prototype;
/**
 * Create a dependency resolver by providing a callback to extract requests,
 * and another callbck to resolve such requests.
 */
function createDependencyResolver({ extractRequests, resolveRequest, }) {
    return (assetKey, deep) => {
        const resolvedAssets = Object.create(null);
        const assetsToResolve = Array.isArray(assetKey) ? [...assetKey] : [assetKey];
        while (assetsToResolve.length > 0) {
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            const currentAsset = assetsToResolve.shift();
            if (resolvedAssets[currentAsset]) {
                continue;
            }
            const resolvedRequests = Object.create(null);
            resolvedAssets[currentAsset] = resolvedRequests;
            const assetRequests = extractRequests(currentAsset);
            for (const request of assetRequests) {
                if (hasOwnProperty.call(resolvedRequests, request)) {
                    continue; // already resolved this request
                }
                const resolvedRequest = resolveRequest(currentAsset, request);
                resolvedRequests[request] = resolvedRequest;
                if (deep && resolvedRequest !== undefined && resolvedRequest !== false) {
                    assetsToResolve.push(resolvedRequest);
                }
            }
        }
        return resolvedAssets;
    };
}
exports.createDependencyResolver = createDependencyResolver;
//# sourceMappingURL=dependency-resolver.js.map