"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.syncToAsyncFs = void 0;
function syncToAsyncFs(syncFs) {
    return {
        ...syncFs,
        watchService: syncFs.watchService,
        caseSensitive: syncFs.caseSensitive,
        promises: {
            readFile: async function readFile(...args) {
                return syncFs.readFileSync(...args);
            },
            async writeFile(...args) {
                return syncFs.writeFileSync(...args);
            },
            async unlink(filePath) {
                return syncFs.unlinkSync(filePath);
            },
            readdir: async function readdir(...args) {
                return syncFs.readdirSync(...args);
            },
            async mkdir(directoryPath, ...args) {
                return syncFs.mkdirSync(directoryPath, ...args);
            },
            async rmdir(directoryPath) {
                return syncFs.rmdirSync(directoryPath);
            },
            async exists(nodePath) {
                return syncFs.existsSync(nodePath);
            },
            async stat(nodePath) {
                return syncFs.statSync(nodePath);
            },
            async lstat(nodePath) {
                return syncFs.lstatSync(nodePath);
            },
            async realpath(nodePath) {
                return syncFs.realpathSync(nodePath);
            },
            async rename(srcPath, destPath) {
                return syncFs.renameSync(srcPath, destPath);
            },
            async copyFile(...args) {
                return syncFs.copyFileSync(...args);
            },
            async readlink(path) {
                return syncFs.readlinkSync(path);
            },
            async symlink(...args) {
                return syncFs.symlinkSync(...args);
            },
            async rm(...args) {
                return syncFs.rmSync(...args);
            },
            async chmod(...args) {
                return syncFs.chmodSync(...args);
            },
        },
        exists(nodePath, callback) {
            callback(syncFs.existsSync(nodePath));
        },
        readFile: callbackify(syncFs.readFileSync),
        writeFile: callbackify(syncFs.writeFileSync),
        copyFile: callbackify(syncFs.copyFileSync),
        unlink: callbackify(syncFs.unlinkSync),
        readdir: callbackify(syncFs.readdirSync),
        mkdir: callbackify(syncFs.mkdirSync),
        rmdir: callbackify(syncFs.rmdirSync),
        stat: callbackify(syncFs.statSync),
        lstat: callbackify(syncFs.lstatSync),
        realpath: callbackify(syncFs.realpathSync),
        rename: callbackify(syncFs.renameSync),
        readlink: callbackify(syncFs.readlinkSync),
    };
}
exports.syncToAsyncFs = syncToAsyncFs;
function callbackify(fn) {
    return (...args) => {
        const callback = args.pop();
        if (typeof callback !== 'function') {
            throw new Error('callback is not a function');
        }
        try {
            const result = fn(...args);
            callback(null, result);
        }
        catch (e) {
            callback(e);
        }
    };
}
//# sourceMappingURL=sync-to-async-fs.js.map