"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DiagnosticsManager = void 0;
const logger_1 = require("./logger");
const report_diagnostics_1 = require("./report-diagnostics");
class DiagnosticsManager {
    constructor(options = {}) {
        var _a;
        this.options = options;
        this.store = new Map();
        this.log = (_a = this.options.log) !== null && _a !== void 0 ? _a : (0, logger_1.createDefaultLogger)();
    }
    clear() {
        this.store = new Map();
    }
    set(identifier, filepath, processDiagnostics) {
        if (this.store.has(identifier) && filepath && processDiagnostics) {
            this.store.get(identifier).set(filepath, processDiagnostics);
        }
        else {
            this.store.set(identifier, new Map(filepath && processDiagnostics ? [[filepath, processDiagnostics]] : undefined));
        }
    }
    get(identifier, filepath) {
        var _a;
        if (filepath) {
            return (_a = this.store.get(identifier)) === null || _a === void 0 ? void 0 : _a.get(filepath);
        }
        else {
            return this.store.get(identifier);
        }
    }
    delete(identifier, filepath) {
        var _a;
        if (filepath) {
            (_a = this.store.get(identifier)) === null || _a === void 0 ? void 0 : _a.delete(filepath);
        }
        else {
            this.store.delete(identifier);
        }
    }
    report() {
        var _a, _b, _c, _d;
        let diagnosticMode = 'loose';
        const diagnosticMessages = new Map();
        const collectedDiagnostics = new Map();
        for (const buildDiagnostics of this.store.values()) {
            for (const [filePath, { diagnostics, diagnosticsMode: currentMode },] of buildDiagnostics) {
                if (diagnosticMode !== 'strict') {
                    diagnosticMode = currentMode || diagnosticMode;
                }
                if (!diagnosticMessages.has(filePath)) {
                    diagnosticMessages.set(filePath, []);
                    collectedDiagnostics.set(filePath, new Map());
                }
                const currentDiagnostics = diagnosticMessages.get(filePath);
                const ids = collectedDiagnostics.get(filePath);
                for (const diagnostic of diagnostics) {
                    const diagnosticId = `${diagnostic.severity};${diagnostic.message}`;
                    if (!ids.has(diagnosticId)) {
                        ids.set(diagnosticId, diagnostic);
                        currentDiagnostics.push(ids.get(diagnosticId));
                    }
                }
            }
        }
        (_b = (_a = this.options.hooks) === null || _a === void 0 ? void 0 : _a.preReport) === null || _b === void 0 ? void 0 : _b.call(_a, diagnosticMessages);
        if (diagnosticMessages.size) {
            const hasFatalDiagnostics = (0, report_diagnostics_1.reportDiagnostics)(this.log, diagnosticMessages, diagnosticMode);
            (_d = (_c = this.options.hooks) === null || _c === void 0 ? void 0 : _c.postReport) === null || _d === void 0 ? void 0 : _d.call(_c, diagnosticMessages, hasFatalDiagnostics);
        }
        return Boolean(diagnosticMessages.size);
    }
}
exports.DiagnosticsManager = DiagnosticsManager;
//# sourceMappingURL=diagnostics-manager.js.map