"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.reportDiagnostics = void 0;
const logger_1 = require("./logger");
function reportDiagnostics(log, diagnosticsMessages, diagnosticsMode) {
    let message = '[Stylable Diagnostics]';
    for (const [filePath, diagnostics] of diagnosticsMessages.entries()) {
        message += `\n[${filePath}]\n${diagnostics
            .sort(({ offset: a = 0 }, { offset: b = 0 }) => a - b)
            .map(({ code, severity, message }) => `[${severity}: ${code}]: ${message}`)
            .join('\n')}`;
    }
    log(message, logger_1.levels.info);
    return diagnosticsMode === 'strict' && hasErrorOrWarning(diagnosticsMessages);
}
exports.reportDiagnostics = reportDiagnostics;
function hasErrorOrWarning(diagnosticsMessages) {
    for (const diagnostics of diagnosticsMessages.values()) {
        const has = diagnostics.some((diagnostic) => diagnostic.severity === 'error' || diagnostic.severity === 'warning');
        if (has) {
            return true;
        }
    }
    return false;
}
//# sourceMappingURL=report-diagnostics.js.map