"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getAlias = exports.getSourcePath = exports.sourcePathDiagnostics = exports.mergeRules = exports.utilDiagnostics = exports.isValidDeclaration = void 0;
const path_1 = require("path");
const diagnostics_1 = require("./diagnostics");
const rule_1 = require("./helpers/rule");
const selector_1 = require("./helpers/selector");
function isValidDeclaration(decl) {
    return typeof decl.value === 'string';
}
exports.isValidDeclaration = isValidDeclaration;
exports.utilDiagnostics = {
    INVALID_MERGE_OF: (0, diagnostics_1.createDiagnosticReporter)('14001', 'error', (mergeValue) => `invalid merge of: \n"${mergeValue}"`),
};
// ToDo: move to helpers/mixin
function mergeRules(mixinAst, rule, mixinDecl, report) {
    let mixinRoot = null;
    const nestedInKeyframes = (0, rule_1.isChildOfAtRule)(rule, `keyframes`);
    mixinAst.walkRules((mixinRule) => {
        if ((0, rule_1.isChildOfAtRule)(mixinRule, 'keyframes')) {
            return;
        }
        if (mixinRule.selector === '&' && !mixinRoot) {
            if (mixinRule.parent === mixinAst) {
                mixinRoot = mixinRule;
            }
            else {
                const { selector } = (0, selector_1.scopeNestedSelector)((0, selector_1.parseSelectorWithCache)(rule.selector), (0, selector_1.parseSelectorWithCache)(mixinRule.selector));
                mixinRoot = 'NoRoot';
                mixinRule.selector = selector;
            }
        }
        else {
            const { selector } = (0, selector_1.scopeNestedSelector)((0, selector_1.parseSelectorWithCache)(rule.selector), (0, selector_1.parseSelectorWithCache)(mixinRule.selector));
            mixinRule.selector = selector;
        }
    });
    if (mixinAst.nodes) {
        let nextRule = rule;
        // TODO: handle rules before and after decl on entry
        mixinAst.nodes.slice().forEach((node) => {
            if (node === mixinRoot) {
                node.walkDecls((node) => {
                    rule.insertBefore(mixinDecl, node);
                });
            }
            else if (node.type === 'decl') {
                rule.insertBefore(mixinDecl, node);
            }
            else if (node.type === 'rule' || node.type === 'atrule') {
                const valid = !nestedInKeyframes;
                if (valid) {
                    if (rule.parent.last === nextRule) {
                        rule.parent.append(node);
                    }
                    else {
                        rule.parent.insertAfter(nextRule, node);
                    }
                    nextRule = node;
                }
                else {
                    report === null || report === void 0 ? void 0 : report.report(exports.utilDiagnostics.INVALID_MERGE_OF(node.toString()), {
                        node: rule,
                    });
                }
            }
        });
    }
    return rule;
}
exports.mergeRules = mergeRules;
exports.sourcePathDiagnostics = {
    MISSING_SOURCE_FILENAME: (0, diagnostics_1.createDiagnosticReporter)('17001', 'error', () => 'missing source filename'),
};
function getSourcePath(root, diagnostics) {
    const source = (root.source && root.source.input.file) || '';
    if (!source) {
        diagnostics.report(exports.sourcePathDiagnostics.MISSING_SOURCE_FILENAME(), {
            node: root,
        });
    }
    else if (!(0, path_1.isAbsolute)(source)) {
        throw new Error('source filename is not absolute path: "' + source + '"');
    }
    return source;
}
exports.getSourcePath = getSourcePath;
function getAlias(symbol) {
    if (symbol._kind === 'class' || symbol._kind === 'element') {
        if (!symbol[`-st-extends`]) {
            return symbol.alias;
        }
    }
    return undefined;
}
exports.getAlias = getAlias;
//# sourceMappingURL=stylable-utils.js.map