"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateStylableJSModuleSource = void 0;
function generateStylableJSModuleSource(moduleOptions, injectOptions) {
    const { namespace, imports = [], jsExports, moduleType, runtimeRequest, varType = 'const', header = '', footer = '', } = moduleOptions;
    const { classes, keyframes, layers, containers, stVars, vars } = jsExports;
    const exportKind = moduleType === 'esm' ? `export ${varType} ` : 'module.exports.';
    return `
${imports.map(moduleRequest(moduleType)).join('\n')}
${runtimeImport(moduleType, runtimeRequest, injectOptions)}

${header}

${varType} _namespace_ = ${JSON.stringify(namespace)};
${varType} _style_ = /*#__PURE__*/ classesRuntime.bind(null, _namespace_);

${exportKind}cssStates = /*#__PURE__*/ statesRuntime.bind(null, _namespace_);
${exportKind}style = /*#__PURE__*/ _style_;
${exportKind}st = /*#__PURE__*/ _style_;

${exportKind}namespace = _namespace_;
${exportKind}classes = ${JSON.stringify(classes)};
${exportKind}keyframes = ${JSON.stringify(keyframes)}; 
${exportKind}layers = ${JSON.stringify(layers)};
${exportKind}containers = ${JSON.stringify(containers)};
${exportKind}stVars = ${JSON.stringify(stVars)}; 
${exportKind}vars = ${JSON.stringify(vars)}; 

${runtimeExecuteInject(moduleType, injectOptions)}

${footer}
`;
}
exports.generateStylableJSModuleSource = generateStylableJSModuleSource;
function moduleRequest(moduleType) {
    return (moduleRequest) => {
        const request = JSON.stringify(moduleRequest.from);
        return moduleType === 'esm' ? `import ${request};` : `require(${request});`;
    };
}
function runtimeImport(moduleType, runtimeRequest, injectOptions) {
    const importInjectCSS = (injectOptions === null || injectOptions === void 0 ? void 0 : injectOptions.css) ? `, injectCSS` : '';
    const request = JSON.stringify(runtimeRequest !== null && runtimeRequest !== void 0 ? runtimeRequest : 
    // TODO: we use direct requests here since we don't know how this will be resolved
    (moduleType === 'esm'
        ? '@stylable/runtime/esm/runtime.js'
        : '@stylable/runtime/dist/runtime.js'));
    return moduleType === 'esm'
        ? `import { classesRuntime, statesRuntime${importInjectCSS} } from ${request};`
        : `const { classesRuntime, statesRuntime${importInjectCSS} } = require(${request});`;
}
function runtimeExecuteInject(moduleType, injectOptions) {
    if (!(injectOptions === null || injectOptions === void 0 ? void 0 : injectOptions.css)) {
        return '';
    }
    const { id, css, cssCode, depthCode, depth, runtimeId } = injectOptions;
    let out = 'injectCSS(';
    out += id ? JSON.stringify(id) : moduleType === 'esm' ? 'import.meta.url' : 'module.id';
    out += ', ';
    out += cssCode || JSON.stringify(css);
    out += ', ';
    out += depthCode || JSON.stringify(depth) || '-1';
    out += ', ';
    out += JSON.stringify(runtimeId);
    out += ');';
    return out;
}
//# sourceMappingURL=stylable-js-module-source.js.map