"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.bundleServerLibs = exports.applyWebpackConfigStylableExcludes = void 0;
function applyWebpackConfigStylableExcludes(webpackConfig) {
    var _a;
    safelyWalkJSON((_a = webpackConfig.module) !== null && _a !== void 0 ? _a : {}, insertStCssExclude(/\.st\.css$/));
}
exports.applyWebpackConfigStylableExcludes = applyWebpackConfigStylableExcludes;
function insertStCssExclude(stRegex) {
    return (_, value) => {
        var _a, _b;
        if (typeof value !== 'string' && value && (value.test || value.issuer)) {
            if (((_a = value.test) === null || _a === void 0 ? void 0 : _a.toString().includes('css')) ||
                ((_b = value.issuer) === null || _b === void 0 ? void 0 : _b.toString().includes('css'))) {
                if (value.exclude && Array.isArray(value.exclude)) {
                    value.exclude.push(stRegex);
                    return false;
                }
                else if (value.exclude instanceof RegExp) {
                    value.exclude = [value.exclude, stRegex];
                    return false;
                }
                else if (typeof value.exclude === 'function') {
                    value.exclude = [value.exclude, stRegex];
                    return false;
                }
                else if (value.exclude) {
                    throw new Error('unknown exclude pattern: ' + value.exclude);
                }
                else {
                    value.exclude = [stRegex];
                    return false;
                }
            }
        }
        return;
    };
}
function safelyWalkJSON(obj, visitor, path = [], visited = new Set()) {
    for (const key in obj) {
        const currentPath = [...path, key];
        if (visited.has(obj[key])) {
            continue;
        }
        else {
            visited.add(obj[key]);
        }
        const res = visitor(key, obj[key], currentPath);
        if (res === false) {
            continue;
        }
        if (typeof obj[key] === 'object') {
            safelyWalkJSON(obj[key], visitor, currentPath, visited);
        }
    }
}
function bundleServerLibs(config, packages, isServer) {
    let hasError = false;
    if (isServer) {
        if (Array.isArray(config.externals) && config.externals.length === 1) {
            const nextExternal = config.externals[0];
            if (typeof nextExternal === 'function') {
                config.externals = [
                    async (ctx, cb) => {
                        for (const pack of packages) {
                            if (ctx.request.startsWith(pack)) {
                                return false;
                            }
                        }
                        return nextExternal(ctx, cb);
                    },
                ];
            }
            else {
                hasError = true;
            }
        }
        else {
            hasError = true;
        }
        if (hasError) {
            throw new Error('Invalid configuration: expected config.externals to be an Array with a single function. got ' +
                JSON.stringify(config.externals));
        }
    }
}
exports.bundleServerLibs = bundleServerLibs;
//# sourceMappingURL=webpack-config-stylable-excludes.js.map