"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCartesianProduct = exports.swap = exports.randomizedOrder = exports.shuffle = void 0;
/**
 * Shuffles an array
 * @returns a new array with the same elements in a random order
 */
function shuffle(array) {
    const order = randomizedOrder(array.length);
    return array.map((_, i) => array[order[i]]);
}
exports.shuffle = shuffle;
/**
 *
 * @returns an array of integers in range [0..size-1] in random order
 */
function randomizedOrder(size) {
    const arr = new Array(size).fill(0).map((_, i) => i);
    for (let i = arr.length - 1; i > 0; i--) {
        const j = Math.floor(Math.random() * (i + 1));
        swap(arr, i, j);
    }
    return arr;
}
exports.randomizedOrder = randomizedOrder;
/**
 * Swaps elements of an array in place
 */
function swap(array, i, j) {
    const temp = array[i];
    array[i] = array[j];
    array[j] = temp;
}
exports.swap = swap;
/**
 * @example
 * ```ts
 * getCartesianProduct([
                [1, 2],
                [3, 4],
            ]); // => [[1, 3], [1, 4], [2, 3], [2, 4]]
 * ```
 * @returns an array containing all the combinations of one element from each array
 */
function getCartesianProduct(arrays) {
    if (arrays.length === 0) {
        return [];
    }
    else if (arrays.length === 1) {
        return arrays[0].map((elem) => [elem]);
    }
    else {
        const otherCombinations = getCartesianProduct(arrays.slice(1));
        const finalCombinations = [];
        for (const elem of arrays[0]) {
            for (const combo of otherCombinations) {
                finalCombinations.push([elem, ...combo]);
            }
        }
        return finalCombinations;
    }
}
exports.getCartesianProduct = getCartesianProduct;
//# sourceMappingURL=arrays.js.map