import { at, concat, every, filter, find, first, flat, flatMap, forEach, includes, isEmpty, join, last, map, next, prev, reduce, size, skip, some, sort, unique, } from './iterables';
import { mapValues } from './objects';
export function chain(value) {
    const iterable = (value === undefined
        ? []
        : value === null
            ? [null]
            : typeof value === 'object' && Symbol.iterator in value
                ? value
                : [value]);
    return iterable === value ? chainIter(iterable) : chainElement(value);
}
function chainIter(iterable) {
    const toIter = { skip, map, flatMap, filter, concat, flat, unique, sort };
    const toElm = { join, last, first, isEmpty, size, at, next, prev, find, some, includes, every, reduce };
    const boundToIter = mapValues(toIter, (v) => (...args) => chainIter(v(iterable, ...args)));
    const boundToElm = mapValues(toElm, (v) => (...args) => chainElement(v(iterable, ...args)));
    return {
        value: iterable,
        iterable,
        ...boundToIter,
        ...boundToElm,
        forEach: (mapping) => {
            forEach(iterable, mapping);
            return chainIter(iterable);
        },
    };
}
function chainElement(value) {
    const iterable = (value === undefined ? [] : [value]);
    return {
        ...chainIter(iterable),
        value,
    };
}
//# sourceMappingURL=chain.js.map