"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WsClientHost = void 0;
const socket_io_client_1 = require("socket.io-client");
const base_host_1 = require("./base-host");
const patterns_1 = require("@wixc3/patterns");
const promise_assist_1 = require("promise-assist");
class WsClientHost extends base_host_1.BaseHost {
    constructor(url, options) {
        super();
        this.subscribers = new patterns_1.EventEmitter();
        const { path, ...query } = Object.fromEntries(new URL(url).searchParams);
        const { promise, resolve } = (0, promise_assist_1.deferred)();
        this.connected = promise;
        this.socketClient = (0, socket_io_client_1.io)(url, {
            transports: ['websocket'],
            forceNew: true,
            withCredentials: true,
            path,
            query,
            ...options,
        });
        this.socketClient.on('connect', () => {
            this.socketClient.on('message', (data) => {
                this.emitMessageHandlers(data);
            });
            resolve();
        });
        this.socketClient.on('disconnect', () => {
            this.subscribers.emit('disconnect', undefined);
            this.socketClient.close();
        });
        this.socketClient.on('reconnect', () => {
            this.subscribers.emit('reconnect', undefined);
        });
        this.socketClient.connect();
    }
    postMessage(data) {
        this.socketClient.emit('message', data);
    }
    dispose() {
        this.subscribers.clear();
        this.socketClient.close();
    }
}
exports.WsClientHost = WsClientHost;
//# sourceMappingURL=ws-client-host.js.map