"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const base_host_1 = require("./com/hosts/base-host");
const communication_1 = require("./com/communication");
const logger_service_1 = require("./com/logger-service");
const config_1 = require("./entities/config");
const env_1 = require("./entities/env");
const feature_1 = require("./entities/feature");
const value_1 = require("./entities/value");
const slot_1 = require("./entities/slot");
const symbols_1 = require("./symbols");
const types_1 = require("./types");
const window_initializer_service_1 = require("./com/window-initializer-service");
exports.default = new feature_1.Feature({
    id: 'COM',
    api: {
        config: config_1.Config.withType().defineEntity({
            id: '',
            loggerSeverity: types_1.LogLevel.DEBUG,
            maxLogMessages: 100,
            topology: {},
            resolvedContexts: {},
        }, (a, b) => ({
            ...a,
            ...b,
            topology: {
                ...a.topology,
                ...b.topology,
            },
            resolvedContexts: {
                ...a.resolvedContexts,
                ...b.resolvedContexts,
            },
            connectedEnvironments: {
                ...a.connectedEnvironments,
                ...b.connectedEnvironments,
            },
        })),
        loggerTransports: slot_1.Slot.withType().defineEntity(env_1.Universal),
        loggerService: value_1.Value.withType().defineEntity(env_1.Universal),
        communication: value_1.Value.withType().defineEntity(env_1.AllEnvironments),
    },
}).setup(env_1.Universal, ({ config: { host, id, topology, maxLogMessages, loggerSeverity, logToConsole, resolvedContexts, publicPath, connectedEnvironments = {}, }, loggerTransports, [symbols_1.RUN_OPTIONS]: runOptions, [symbols_1.ENGINE]: engine, onDispose, }) => {
    var _a;
    const isNode = typeof process !== 'undefined' &&
        !!((_a = process.versions) === null || _a === void 0 ? void 0 : _a.node) &&
        process.title !== 'browser' &&
        // in electron process also have type 'renderer'
        process.type !== 'renderer';
    // iframe gets `instanceId` with top level config
    // webworker gets `instanceId` set into `name` property when initialized as Environment.
    // it can be overridden using top level config.
    // main frame might not have that configured, so we use 'main' fallback for it.
    const comId = id || (host && host.name) || (typeof self !== 'undefined' && self.name) || engine.entryEnvironment.env;
    const comOptions = {
        warnOnSlow: runOptions.has('warnOnSlow'),
        publicPath,
        connectedEnvironments,
    };
    const communication = new communication_1.Communication(isNode ? host || new base_host_1.BaseHost() : self, comId, topology, resolvedContexts, isNode, comOptions);
    // manually register window initialization api service to be used during
    // start of managed iframe in packages/core/src/com/initializers/iframe.ts
    communication.registerAPI({ id: window_initializer_service_1.WindowInitializerService.apiId }, new window_initializer_service_1.WindowInitializerService());
    const loggerService = new logger_service_1.LoggerService(loggerTransports, { environment: communication.getEnvironmentId() }, { severity: loggerSeverity, maxLogMessages, logToConsole });
    onDispose(() => communication.dispose());
    return {
        loggerService,
        communication,
    };
});
//# sourceMappingURL=communication.feature.js.map