"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.testEnvironmentCollision = exports.normEnvVisibility = exports.SingleEndpointContextualEnvironment = exports.orderedEnvDependencies = exports.globallyProvidingEnvironments = exports.AllEnvironments = exports.Universal = exports.EnvironmentContext = exports.Environment = void 0;
const entity_helpers_1 = require("../entity-helpers");
class Environment {
    constructor(env, envType, endpointType, dependencies = []) {
        this.env = env;
        this.envType = envType;
        this.endpointType = endpointType;
        this.dependencies = dependencies;
    }
}
exports.Environment = Environment;
class EnvironmentContext {
    constructor(env, activeEnvironmentName) {
        this.env = env;
        this.activeEnvironmentName = activeEnvironmentName;
    }
}
exports.EnvironmentContext = EnvironmentContext;
exports.Universal = new Environment('<Universal>', 'window', 'multi');
exports.AllEnvironments = new Environment('<All>', 'window', 'multi');
exports.globallyProvidingEnvironments = new Set([exports.Universal.env, exports.AllEnvironments.env]);
function orderedEnvDependencies(env) {
    var _a, _b;
    return (_b = (_a = env.dependencies) === null || _a === void 0 ? void 0 : _a.flatMap(orderedEnvDependencies).concat(env.env)) !== null && _b !== void 0 ? _b : [];
}
exports.orderedEnvDependencies = orderedEnvDependencies;
class SingleEndpointContextualEnvironment extends Environment {
    constructor(env, environments) {
        super(env, 'context', 'single');
        this.environments = environments;
        if (environments.length === 0) {
            throw new Error(`Contextual Environment ${env} initiated without child environments`);
        }
    }
    useContext(contextEnv) {
        return new EnvironmentContext(this.env, contextEnv);
    }
    withContext() {
        return {
            type: (0, entity_helpers_1.runtimeType)(this.env + ' context'),
        };
    }
    getEnvironmentById(id) {
        return this.environments.find(({ env }) => env === id);
    }
}
exports.SingleEndpointContextualEnvironment = SingleEndpointContextualEnvironment;
function normEnvVisibility(envVisibility) {
    const envSet = new Set();
    const extractDependencies = (env) => {
        for (const { env: depEnv, dependencies } of env.dependencies) {
            envSet.add(depEnv);
            dependencies.map(extractDependencies);
        }
    };
    const envs = Array.isArray(envVisibility) ? envVisibility : [envVisibility];
    for (const e of envs) {
        envSet.add(e.env);
        extractDependencies(e);
    }
    return envSet;
}
exports.normEnvVisibility = normEnvVisibility;
function testEnvironmentCollision(envVisibility, envSet) {
    const containsEnv = new Set();
    const test = (env) => {
        envSet.has(env) ? containsEnv.add(env) : envSet.add(env);
    };
    if (Array.isArray(envVisibility)) {
        for (const e of envVisibility) {
            test(e.env);
        }
    }
    else if (typeof envVisibility === 'string') {
        test(envVisibility);
    }
    else {
        test(envVisibility.env);
    }
    return [...containsEnv];
}
exports.testEnvironmentCollision = testEnvironmentCollision;
//# sourceMappingURL=env.js.map