"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Service = void 0;
const communication_feature_1 = __importDefault(require("../communication.feature"));
const symbols_1 = require("../symbols");
const env_1 = require("./env");
const output_1 = require("./output");
class Service extends output_1.FeatureOutput {
    static withType() {
        return {
            defineEntity(providedFrom) {
                return new Service(providedFrom, providedFrom, false);
            },
        };
    }
    constructor(providedFrom, visibleAt, remoteAccess, options = {}) {
        super(providedFrom, visibleAt, remoteAccess);
        this.providedFrom = providedFrom;
        this.visibleAt = visibleAt;
        this.remoteAccess = remoteAccess;
        this.options = options;
    }
    allowRemoteAccess(options) {
        return new Service(this.providedFrom, env_1.AllEnvironments, true, options);
    }
    [symbols_1.REGISTER_VALUE](runtimeEngine, providedValue, inputValue, featureID, entityKey) {
        if (this.remoteAccess) {
            const { communication } = runtimeEngine.get(communication_feature_1.default).api;
            const serviceKey = runtimeEngine.entityID(featureID, entityKey);
            const providedFrom = (0, env_1.normEnvVisibility)(this.providedFrom);
            const localEnv = communication.getEnvironmentName();
            if (providedFrom.has(localEnv) || providedFrom.has(env_1.Universal.env)) {
                if (!providedValue) {
                    throw new Error(`Service is not provided at runtime.
Make sure the environment setup file exists and named correctly: [featureName].[envName].env.[ext]
Service name: ${entityKey}
Feature id: ${featureID}
Environment: ${localEnv}
                        `);
                }
                communication.registerAPI({ id: serviceKey }, providedValue);
                return providedValue;
            }
            // eslint-disable-next-line @typescript-eslint/no-unsafe-return
            return inputValue || this.getApiProxy(runtimeEngine, serviceKey);
        }
        return providedValue;
    }
    [symbols_1.CREATE_RUNTIME](context, featureID, entityKey) {
        if (this.remoteAccess) {
            // eslint-disable-next-line @typescript-eslint/no-unsafe-return
            return this.getApiProxy(context, context.entityID(featureID, entityKey));
        }
    }
    getApiProxy(context, serviceKey) {
        const { communication } = context.get(communication_feature_1.default).api;
        const instanceId = getSingleInstanceId(this.providedFrom);
        if (instanceId) {
            return communication.apiProxy({ id: instanceId }, { id: serviceKey }, this.options);
        }
        else {
            return {
                get: (token) => {
                    return communication.apiProxy(token, { id: serviceKey }, this.options);
                },
            };
        }
    }
}
exports.Service = Service;
function getSingleInstanceId(providedFrom) {
    if (isSingleInstance(providedFrom)) {
        return providedFrom.env;
    }
}
function isSingleInstance(providedFrom) {
    return (!!providedFrom &&
        providedFrom.endpointType &&
        providedFrom.endpointType === 'single');
}
//# sourceMappingURL=service.js.map