"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.initializeNodeEnvironment = void 0;
const child_process_1 = require("child_process");
const engine_core_node_1 = require("@wixc3/engine-core-node");
const tree_kill_1 = __importDefault(require("tree-kill"));
const util_1 = require("util");
const promisifiedTreeKill = (0, util_1.promisify)(tree_kill_1.default);
/**
 * Spawn a node-based environment from the renderer process.
 * Should be invoked *only* from the renderer process.
 * The caller should await returned `environmentIsReady` promise to ensure environment initialization is finished.
 */
const initializeNodeEnvironment = ({ communication, env, runtimeArguments, processOptions, environmentStartupOptions }) => {
    var _a, _b, _c, _d, _e;
    const environmentIsReady = communication.envReady(env.env);
    const nodeEnvStartupArguments = {
        ...runtimeArguments,
        ...environmentStartupOptions,
        outputPath: runtimeArguments.basePath,
        parentEnvName: communication.getEnvironmentName(),
        environmentName: env.env,
        env,
    };
    const child = (0, child_process_1.spawn)((_a = environmentStartupOptions === null || environmentStartupOptions === void 0 ? void 0 : environmentStartupOptions.execPath) !== null && _a !== void 0 ? _a : process.execPath, ['--unhandled-rejections=strict', runtimeArguments.nodeEntryPath], {
        stdio: ['pipe', 'pipe', 'pipe', 'ipc'],
        ...processOptions,
        detached: true,
        env: {
            ...process.env,
            ...processOptions === null || processOptions === void 0 ? void 0 : processOptions.env,
            // Spawn as a regular Node process. Without this option environment
            // communication fails quietly for reasons unknown.
            ELECTRON_RUN_AS_NODE: 'true',
        },
    });
    child.send({ id: 'nodeStartupOptions', runOptions: nodeEnvStartupArguments });
    const host = new engine_core_node_1.IPCHost(child);
    communication.registerEnv(env.env, host);
    communication.registerMessageHandler(host);
    // Pipe stdout and stderr to console in the renderer process.
    (_b = child.stdout) === null || _b === void 0 ? void 0 : _b.setEncoding('utf8');
    (_c = child.stderr) === null || _c === void 0 ? void 0 : _c.setEncoding('utf8');
    (_d = child.stdout) === null || _d === void 0 ? void 0 : _d.on('data', console.log); // eslint-disable-line no-console
    (_e = child.stderr) === null || _e === void 0 ? void 0 : _e.on('data', console.error); // eslint-disable-line no-console
    return {
        id: env.env,
        // eslint-disable-next-line @typescript-eslint/no-misused-promises
        dispose: async () => {
            if (!child.killed) {
                child.pid ? await promisifiedTreeKill(child.pid) : child.kill();
            }
        },
        onDisconnect: (cb) => {
            child.on('exit', cb);
        },
        environmentIsReady,
    };
};
exports.initializeNodeEnvironment = initializeNodeEnvironment;
//# sourceMappingURL=electron-node.js.map