"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const engine_core_1 = require("@wixc3/engine-core");
const engine_core_node_1 = require("@wixc3/engine-core-node");
const engine_runtime_node_1 = require("@wixc3/engine-runtime-node");
const import_modules_1 = require("./import-modules");
const types_1 = require("./types");
const onMessageListener = async (message) => {
    if ((0, types_1.isNodeEnvStartupMessage)(message)) {
        const { requiredModules, basePath, environmentName, config, environmentContextName, featureName, features, bundlePath, runtimeOptions, parentEnvName, env, } = message.runOptions;
        if (requiredModules) {
            await (0, import_modules_1.importModules)(basePath, requiredModules);
        }
        // if current node environment wishes to launch a new one, it needs to pass on the runtime arguments it received.
        // creating an access point at runtime application, so it could use the ENGINE_PARENT_ENV_ID to be able to retrieve all getRuntimeArguments values into the app while launching a new environment using the initializer provided from '@wixc3/engine-electron-node'
        const parentHost = new engine_core_1.BaseHost();
        const com = new engine_core_1.Communication(parentHost, engine_core_node_1.LOCAL_ENVIRONMENT_INITIALIZER_ENV_ID);
        com.registerAPI(types_1.metadataApiToken, {
            getRuntimeArguments: () => message.runOptions,
        });
        const comHost = parentHost.open();
        com.registerEnv(environmentName, comHost);
        config.push(engine_core_1.COM.use({
            config: {
                connectedEnvironments: {
                    [engine_core_node_1.LOCAL_ENVIRONMENT_INITIALIZER_ENV_ID]: {
                        id: engine_core_node_1.LOCAL_ENVIRONMENT_INITIALIZER_ENV_ID,
                        host: comHost,
                    },
                },
            },
        }));
        await (0, engine_runtime_node_1.runIPCEnvironment)({
            type: 'node',
            name: environmentName,
            bundlePath,
            childEnvName: environmentContextName,
            featureName,
            config,
            features,
            options: runtimeOptions,
            context: basePath,
            parentEnvName,
            env,
        });
    }
};
// eslint-disable-next-line @typescript-eslint/no-misused-promises
process.once('message', onMessageListener);
//# sourceMappingURL=node-entry.js.map