"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = __importDefault(require("chai"));
const chai_as_promised_1 = __importDefault(require("chai-as-promised"));
const node_1 = __importDefault(require("@file-services/node"));
const engine_core_1 = require("@wixc3/engine-core");
const engine_scripts_1 = require("@wixc3/engine-scripts");
const engine_electron_commons_1 = require("@wixc3/engine-electron-commons");
const test_feature_feature_1 = __importStar(require("../test-project/test-feature.feature"));
const { expect } = chai_1.default;
chai_1.default.use(chai_as_promised_1.default);
const testProjectPath = node_1.default.join(__dirname, '../test-project');
const setupRunningEnv = async ({ errorMode, handleUncaught, } = {}) => {
    const communication = new engine_core_1.Communication(new engine_core_1.BaseHost(), 'someId');
    const { features } = (0, engine_scripts_1.findFeatures)(testProjectPath, node_1.default, 'dist');
    const { onDisconnect, dispose, environmentIsReady } = (0, engine_electron_commons_1.initializeNodeEnvironment)({
        communication,
        env: test_feature_feature_1.serverEnv,
        runtimeArguments: {
            featureName: test_feature_feature_1.default.id,
            basePath: testProjectPath,
            outputPath: node_1.default.join(testProjectPath, 'dist-app'),
            nodeEntryPath: node_1.default.join(testProjectPath, 'entry.js'),
            config: [test_feature_feature_1.default.use({ errorType: { type: errorMode, handleUncaught } })],
            features: Array.from(features.entries()),
        },
        processOptions: { cwd: process.cwd(), stdio: ['ignore', 'ignore', 'ignore', 'ipc'] },
        environmentStartupOptions: {},
    });
    await environmentIsReady;
    const disconnectPromise = new Promise((res) => {
        onDisconnect(() => {
            res(true);
        });
    });
    return {
        onDisconnect,
        dispose,
        disconnectPromise,
    };
};
describe('onDisconnectHandler for node environment initializer', () => {
    describe('without own uncaughtException handling', () => {
        it('should catch on dispose of env', async () => {
            const { dispose, disconnectPromise } = await setupRunningEnv();
            dispose();
            await expect(disconnectPromise).to.eventually.eq(true);
        });
        it('should catch on env exit intentionally', async () => {
            const { disconnectPromise } = await setupRunningEnv({ errorMode: 'exit' });
            await expect(disconnectPromise).to.eventually.eq(true);
        });
        it('should catch on env throwing uncaught exception', async () => {
            const { disconnectPromise } = await setupRunningEnv({ errorMode: 'exception' });
            await expect(disconnectPromise).to.eventually.eq(true);
        });
        it('should catch on env unhandled promise rejection', async () => {
            const { disconnectPromise } = await setupRunningEnv({ errorMode: 'promiseReject' });
            await expect(disconnectPromise).to.eventually.eq(true);
        });
    });
    describe('with own uncaughtException handling', () => {
        const handleUncaught = true;
        it('should catch on dispose of env', async () => {
            const { dispose, disconnectPromise } = await setupRunningEnv({ handleUncaught });
            dispose();
            await expect(disconnectPromise).to.eventually.eq(true);
        });
        it('should catch on env exit intentionally', async () => {
            const { disconnectPromise } = await setupRunningEnv({ errorMode: 'exit', handleUncaught });
            await expect(disconnectPromise).to.eventually.eq(true);
        });
        it('should catch on env throwing uncaught exception', async () => {
            const { disconnectPromise } = await setupRunningEnv({ errorMode: 'exception', handleUncaught });
            await expect(disconnectPromise).to.eventually.eq(true);
        });
        it('should catch on env unhandled promise rejection', async () => {
            const { disconnectPromise } = await setupRunningEnv({ errorMode: 'promiseReject', handleUncaught });
            await expect(disconnectPromise).to.eventually.eq(true);
        });
    });
});
//# sourceMappingURL=electron-node.spec.js.map