"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.runElectronEnv = void 0;
const node_1 = __importDefault(require("@file-services/node"));
const engine_core_1 = require("@wixc3/engine-core");
const engine_electron_commons_1 = require("@wixc3/engine-electron-commons");
const import_modules_1 = require("@wixc3/engine-electron-commons/dist/import-modules");
const engine_runtime_node_1 = require("@wixc3/engine-runtime-node");
const electron_1 = require("electron");
const runtime_arguments_provider_1 = __importDefault(require("./runtime-arguments-provider"));
const nodeEntryPath = require.resolve('@wixc3/engine-electron-commons/node-entry');
async function runElectronEnv({ basePath, outputPath = node_1.default.join(basePath, 'dist-app'), configName, featureName, devport, devtools, features, config, requiredModules, env, }) {
    if (requiredModules) {
        await (0, import_modules_1.importModules)(basePath, requiredModules);
    }
    if (!featureName) {
        throw new Error('Must provide a featureName');
    }
    if (!electron_1.app.isReady()) {
        await new Promise((resolve) => electron_1.app.once('ready', resolve));
    }
    /****************
     Now, after application is ready, running the electron-main environment
     ****************/
    // creating a communication channel, that will be used for node environment initialization
    const generalConfig = [...config];
    const runOptions = new Map();
    const getRuntimeArguments = () => {
        return Promise.resolve({
            basePath,
            featureName,
            outputPath,
            configName,
            nodeEntryPath,
            devtools,
            devport,
            features: Array.from(features.entries()),
            config: generalConfig,
            requiredModules,
            runtimeOptions: Array.from(runOptions.entries()),
        });
    };
    runtime_arguments_provider_1.default.setProvider(getRuntimeArguments);
    electron_1.ipcMain.handle(engine_electron_commons_1.communicationChannels.engineRuntimeArguments, () => runtime_arguments_provider_1.default.getRuntimeArguments());
    // creating runOptions for the electron host environment. ipcMain cannot invoke events from itself, only ipcRenderer can, so we cannot use communicationChannels.engineRuntimeArguments
    runOptions.set(engine_electron_commons_1.electronRuntimeArguments.runtimeFeatureName, featureName);
    runOptions.set(engine_electron_commons_1.electronRuntimeArguments.devtools, !!devtools);
    if (configName) {
        runOptions.set(engine_electron_commons_1.electronRuntimeArguments.runtimeConfigName, configName);
    }
    // in dev mode, we always have to have a port from where the main environment is being served from
    if (devport) {
        runOptions.set(engine_electron_commons_1.electronRuntimeArguments.devport, `${devport}`);
    }
    else {
        runOptions.set(engine_electron_commons_1.electronRuntimeArguments.outPath, basePath);
    }
    if (devtools !== undefined) {
        runOptions.set(engine_electron_commons_1.electronRuntimeArguments.devtools, devtools);
    }
    const runningEnvironment = await (0, engine_runtime_node_1.runNodeEnvironment)({
        featureName,
        features: [...features.entries()],
        name: env.env,
        bundlePath: engine_electron_commons_1.electronRuntimeArguments.outPath,
        type: 'node',
        host: new engine_core_1.BaseHost(),
        config,
        options: [...runOptions.entries()],
        env,
    });
    // eslint-disable-next-line @typescript-eslint/no-misused-promises
    electron_1.app.on('window-all-closed', async () => {
        // Don't quit between closing one window and immediately opening another.
        await new Promise((res) => setTimeout(res, 0));
        if (electron_1.BrowserWindow.getAllWindows().length === 0) {
            try {
                await runningEnvironment.dispose();
            }
            catch (e) {
                process.exitCode = 1;
                // eslint-disable-next-line no-console
                console.error(e);
            }
            electron_1.app.quit();
        }
    });
    return runningEnvironment;
}
exports.runElectronEnv = runElectronEnv;
//# sourceMappingURL=start-electron-env.js.map