"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.initializeWindowEnvironment = exports.windowEnvironmentInitializer = void 0;
const common_1 = require("@wixc3/common");
const engine_electron_commons_1 = require("@wixc3/engine-electron-commons");
const electron_1 = require("electron");
const path_1 = require("path");
const electron_node_host_1 = require("../hosts/electron-node-host");
/**
 * registers a message handler inside the electron main process, which will respond to communication messages coming from the `browserWindow`
 * @param browserWindow the browser winodow that requires communication with the main host
 */
function windowEnvironmentInitializer({ browserWindow, env: { env: envName }, communication: com }) {
    const host = new electron_node_host_1.ElectronBrowserHost(electron_1.ipcMain, browserWindow.webContents);
    com.registerEnv(envName, host);
    com.registerMessageHandler(host);
    return {
        id: envName,
        dispose: (0, common_1.once)(() => {
            host.dispose();
            com.clearEnvironment(envName);
            com.removeMessageHandler(host);
        }),
        browserWindow
    };
}
exports.windowEnvironmentInitializer = windowEnvironmentInitializer;
/**
 * openning a an engine environment in a browser window
 */
async function initializeWindowEnvironment({ env, browserWindow, runOptions, communication, configName = runOptions.get(engine_electron_commons_1.electronRuntimeArguments.runtimeConfigName), featureName = runOptions.get(engine_electron_commons_1.electronRuntimeArguments.runtimeFeatureName), runtimeArguments = {}, }) {
    const devport = runOptions.get(engine_electron_commons_1.electronRuntimeArguments.devport);
    const query = {
        feature: featureName,
        config: configName !== null && configName !== void 0 ? configName : '',
        ...runtimeArguments
    };
    if (devport) {
        const url = new URL(`http://localhost:${devport}/${env.env}.html`);
        for (const [name, val] of Object.entries(query)) {
            url.searchParams.set(name, val);
        }
        await browserWindow.loadURL(url.toString());
    }
    else {
        const basePath = runOptions.get(engine_electron_commons_1.electronRuntimeArguments.outPath);
        const htmlPath = (0, path_1.join)(basePath, `${env.env}.html`);
        await browserWindow.loadFile(htmlPath, { query });
    }
    if (runOptions.get(engine_electron_commons_1.electronRuntimeArguments.devtools)) {
        browserWindow.webContents.openDevTools({ mode: 'bottom' });
    }
    const activeEnvironment = windowEnvironmentInitializer({ env, browserWindow, communication });
    return activeEnvironment;
}
exports.initializeWindowEnvironment = initializeWindowEnvironment;
//# sourceMappingURL=browser-window.js.map