"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.initializeNodeEnvironmentInNode = void 0;
const engine_core_1 = require("@wixc3/engine-core");
const engine_core_node_1 = require("@wixc3/engine-core-node");
const engine_electron_commons_1 = require("@wixc3/engine-electron-commons");
/**
 * Spawn a node-based environment from the renderer process.
 * Should be invoked *only* from the renderer process.
 */
const initializeNodeEnvironmentInNode = async (options) => {
    const runtimeArguments = await getApplicationMetaData(options.communication);
    const { id, dispose, onDisconnect, environmentIsReady } = (0, engine_electron_commons_1.initializeNodeEnvironment)({
        runtimeArguments,
        ...options,
    });
    process.on('exit', dispose);
    await environmentIsReady;
    return { id, onDisconnect };
};
exports.initializeNodeEnvironmentInNode = initializeNodeEnvironmentInNode;
async function getApplicationMetaData(com) {
    const parentHost = com.getEnvironmentHost(engine_core_node_1.LOCAL_ENVIRONMENT_INITIALIZER_ENV_ID);
    if (!parentHost) {
        throw new Error(`no host was initialized under the environment ${engine_core_node_1.LOCAL_ENVIRONMENT_INITIALIZER_ENV_ID}. Cannot retrieve application metadata`);
    }
    if (!parentHost.parent) {
        throw new Error(`the host provided for the ${engine_core_node_1.LOCAL_ENVIRONMENT_INITIALIZER_ENV_ID} environment must be a child of a host of an environment/communication which provides the meta data api`);
    }
    const communication = new engine_core_1.Communication(new engine_core_1.BaseHost(), com.getEnvironmentId());
    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    communication.registerEnv(engine_core_node_1.LOCAL_ENVIRONMENT_INITIALIZER_ENV_ID, parentHost.parent);
    communication.registerMessageHandler(parentHost);
    const apiProxy = communication.apiProxy({
        id: engine_core_node_1.LOCAL_ENVIRONMENT_INITIALIZER_ENV_ID,
    }, engine_electron_commons_1.metadataApiToken);
    return apiProxy.getRuntimeArguments();
}
//# sourceMappingURL=node-process.js.map