# Resource class

A `resource` is a store that manages a piece of immutable state, it allows changing the state and manages subscriptions to its change events.

Engine resources are defined using the [`ResourceFactory`](../readme.md) class, making the constructor arguments here unnecessary, as they will be provided for you.

## Constructor arguments

| name     | type                                  | notes                             |
| -------- | ------------------------------------- | --------------------------------- |
| `id`     | `string`                              | id of the resource [[1]](#Note-1) |
| `config` | [`ResourceOptions`](#ResourceOptions) | see below                         |

### ResourceOptions

| name           | type               | required | notes                                                                                                                                                   |
| -------------- | ------------------ | -------- | ------------------------------------------------------------------------------------------------------------------------------------------------------- |
| `resourceType` | `string`           | true     | id of the resource type                                                                                                                                 |
| `provider`     | `ResourceProvider` | true     | a method returning the initial state of a resource (can be a promise)                                                                                   |
| `reducer`      | `Resource`         | false    | accepts a purposed change and a previous value returning a new immutable value, defaults to a reducer that supports only entire changes to the resource |
| `batcher`      | `Batcher`          | false    | performs multiple changes to multiple resources in a single event                                                                                       |

##### Note-1

> The resource itself does not utilize the provided `id`. However, in the larger context of the engine this `id` will serve as a unique identifier for any resource of the same `type`.

## Lifecycle

### Initialization

Once a resource is created its provider is immediately invoked, if the provider returned a promise the resource is not ready until that promise is resolved.

This is reflected in the resource's API as

- `myResource.isReady()` - returns a `boolean`
- `myResource.ready` - returns a `promise`

### Changes and versions

Once a resource has been declared as ready the following APIs can be used:

- `myResource.getLatest()` - returns the current value of the resource
- `myResource.applyChange(change)` - applies a change to the resource value

Resources also offer an API not just for managing their current state, but also ways of working with changes made across different versions of the resource.

- `myResource.createSnapshot()` - returns a `snapshot` to the current state of the resource. remember to dispose of your snapshots when they are no longer needed as it can adversely affect how history is maintained
- `myResource.getProjection(snapshot)` - returns the value of the resource at the time the snapshot was made
- `myResource.getChangesFromVersion(snapshot)` - returns changes that occurred since the time the snapshot was made
- `myResource.getChangesBetweenVersions(snapshot1, snapshot2)` - returns the changes that occurred between the times in which the snapshots were made

#### Snapshot disposal

When creating a `snapshot`, a pointer is returned. This pointer exposes an API to dispose of it when it is no longer needed.

- `snapshot.dispose()` - disposes of the snapshot, no longer directly maintaining a connection to that point in the resource's history
- `snapshot.isDisposed()` - returns a `boolean` if the snapshot has been disposed of

### Resource disposal

Once a resource in no longer needed, it is important to dispose of it. This will also dispose of all `snapshots` for this resource.

- `myResource.dispose()` - disposes all listeners for this resource
