"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.clear = exports.init = exports.remapToUserLibrary = void 0;
/* eslint-disable prefer-rest-params */
const module_1 = __importDefault(require("module"));
const requestContextProviders = new Set();
/**
 * Add a context provider that will decide, if a request matches it's pattern, the resolution context for that request
 */
function remapToUserLibrary(contextProviderItem) {
    requestContextProviders.add(contextProviderItem);
}
exports.remapToUserLibrary = remapToUserLibrary;
const originalResolveFilename = module_1.default._resolveFilename;
let isMapperRegistered = originalResolveFilename === _resolveFilenameWithMapping;
function _resolveFilenameWithMapping() {
    // using arguments to save cycles. this function is called for every require() or require.resolve() calls
    const [request] = arguments;
    // bails quickly on relative requests to avoid checking map
    if (request && request[0] !== '.') {
        try {
            for (const { test, context } of requestContextProviders) {
                if (test(request)) {
                    module_1.default._resolveFilename = originalResolveFilename;
                    arguments[0] = require.resolve(request, { paths: [context] });
                    break;
                }
            }
        }
        catch {
            /** */
        }
        finally {
            module_1.default._resolveFilename = _resolveFilenameWithMapping;
        }
    }
    return originalResolveFilename.apply(this, arguments);
}
function init() {
    if (!isMapperRegistered && module_1.default._resolveFilename !== _resolveFilenameWithMapping) {
        module_1.default._resolveFilename = _resolveFilenameWithMapping;
        isMapperRegistered = true;
    }
}
exports.init = init;
function clear() {
    if (isMapperRegistered) {
        requestContextProviders.clear();
        module_1.default._resolveFilename = originalResolveFilename;
        isMapperRegistered = false;
    }
}
exports.clear = clear;
//# sourceMappingURL=external-request-mapper.js.map