"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NodeEnvironmentsManager = exports.ChildBaseHost = void 0;
const path_1 = require("path");
const socket_io_1 = __importDefault(require("socket.io"));
const create_listening_server_1 = require("create-listening-server");
const engine_core_1 = require("@wixc3/engine-core");
const remote_node_environment_1 = require("./remote-node-environment");
const ws_environment_1 = require("./ws-environment");
const types_1 = require("./types");
const environments_1 = require("./environments");
const engine_core_node_1 = require("@wixc3/engine-core-node");
class ChildBaseHost extends engine_core_1.BaseHost {
    constructor(parent) {
        super();
        this.parent = parent;
    }
    postMessage(message) {
        this.parent.postMessage(message);
    }
    postToSelf(message) {
        this.emitMessageHandlers(message);
    }
}
exports.ChildBaseHost = ChildBaseHost;
class ChildHostWrapper extends engine_core_1.BaseHost {
    constructor(host) {
        super();
        this.host = host;
    }
    postMessage(message) {
        this.host.postToSelf(message);
    }
}
const cliEntry = require.resolve('./remote-node-entry');
class NodeEnvironmentsManager {
    constructor(socketServer, options, context, socketServerOptions) {
        this.socketServer = socketServer;
        this.options = options;
        this.context = context;
        this.socketServerOptions = socketServerOptions;
        this.runningFeatures = new Map();
    }
    async runServerEnvironments({ featureName, configName, runtimeOptions = {}, overrideConfigsMap = new Map(), mode = 'new-server', }) {
        const runtimeConfigName = configName;
        const featureId = `${featureName}${configName ? path_1.delimiter + configName : ''}`;
        const topology = {};
        const { defaultRuntimeOptions, features } = this.options;
        const featuresWithDefaults = new Map();
        for (const [featureName, featureDef] of features.entries()) {
            featuresWithDefaults.set(featureName, {
                contextFilePaths: {},
                dependencies: [],
                envFilePaths: {},
                exportedEnvs: [],
                preloadFilePaths: {},
                resolvedContexts: {},
                ...featureDef,
            });
        }
        const nodeEnvironments = (0, environments_1.resolveEnvironments)(featureName, featuresWithDefaults, 'node');
        // checking if already has running environments for this feature
        const runningEnv = this.runningFeatures.get(featureId);
        if (runningEnv) {
            // adding the topology of the already running environments for this feature
            Object.assign(topology, this.getTopologyForRunningEnvironments(runningEnv.runningEnvironments));
        }
        const runningEnvironments = {};
        const preparedEnvironments = {};
        /**
         * creating a "top level" communication instance for all running node environments, to serve as a router between them.
         *
         * to this communication all environments will be registered using either a base host which uses 'baseHost' as a parent, either IPCHost (in cases when node environments are launched in forked mode).
         *
         * all node environments will receive a "connectedEnvironments" configuration to the communication feature, to provide access to all node environments. (note: this means that when running engine applications using the Applciation API from engine-scripts, node environments communication is implicit. BUT, we do need to encourage the users to still use the COM's 'initializers' in the 'initiating' environment for both concictensy and correctness.
         *
         * in the other side, when a node environment a wants to communicate with node environment b using this mechanism, it uses the 'localNodeEnvironmentInitializer' exported from '@wixc3/engine-core', as the initializer for communicating with environment b (from a).
         *
         * the communication flow is as follows:
         *
         * 'a' sets up a connection to 'b', using 'localNodeEnvironmentInitializer'.
         * 'a' makes an api call to a service provided from 'b'
         * 'a' locates the host to 'b' and sends the message.
         * message arrives to this top level communication
         * message gets re-routed to 'b'
         * 'b' responds back to top level communication which then forwards the message to 'a'
         *
         * if 'b' wants to call an api provided from 'a', it's implicity will do the same thing, but without the need to explicitly call a COM's initializer
         */
        const baseHost = new engine_core_1.BaseHost();
        const com = new engine_core_1.Communication(baseHost, engine_core_node_1.LOCAL_ENVIRONMENT_INITIALIZER_ENV_ID, undefined, undefined, true);
        const envHostMapping = new Map();
        for (const nodeEnv of nodeEnvironments) {
            const host = new ChildBaseHost(baseHost);
            envHostMapping.set(nodeEnv, host);
            com.registerEnv(nodeEnv.name, new ChildHostWrapper(host));
        }
        for (const nodeEnv of nodeEnvironments) {
            const { overrideConfigs, originalConfigName } = this.getOverrideConfig(overrideConfigsMap, configName, nodeEnv.name);
            const config = [];
            const connectedEnvironments = {};
            for (const [env, host] of envHostMapping) {
                if (env !== nodeEnv) {
                    connectedEnvironments[env.name] = { id: env.name, host };
                }
                else {
                    connectedEnvironments[engine_core_node_1.LOCAL_ENVIRONMENT_INITIALIZER_ENV_ID] = {
                        id: engine_core_node_1.LOCAL_ENVIRONMENT_INITIALIZER_ENV_ID,
                        host,
                        registerMessageHandler: true,
                    };
                }
            }
            config.push(engine_core_1.COM.use({ config: { topology, connectedEnvironments } }));
            config.push(...(await this.getConfig(originalConfigName)), ...overrideConfigs);
            const preparedEnvironment = await this.prepareEnvironment({
                nodeEnv,
                featureName,
                bundlePath: this.options.bundlePath,
                config,
                options: {
                    ...defaultRuntimeOptions,
                    ...runtimeOptions,
                },
                mode,
                com,
                baseHost,
                features: featuresWithDefaults,
            });
            topology[nodeEnv.name] = `http://localhost:${preparedEnvironment.port}/${nodeEnv.name}`;
            preparedEnvironments[nodeEnv.name] = preparedEnvironment;
        }
        for (const [envName, preparedEnvironment] of Object.entries(preparedEnvironments)) {
            if (runningEnvironments[envName]) {
                throw new Error(`${envName} is already running`);
            }
            runningEnvironments[envName] = {
                ...(await preparedEnvironment.start()),
                port: preparedEnvironment.port,
            };
        }
        this.runningFeatures.set(featureId, { com, runningEnvironments });
        return {
            featureName,
            configName: runtimeConfigName,
            runningEnvironments,
        };
    }
    getOverrideConfig(overrideConfigsMap, configName, envName) {
        const { overrideConfig: overrideConfigProvider = [] } = this.options;
        const overrideConfig = Array.isArray(overrideConfigProvider)
            ? overrideConfigProvider
            : envName
                ? overrideConfigProvider(envName)
                : [];
        const overrideConfigs = [...overrideConfig];
        if (configName) {
            const currentOverrideConfig = overrideConfigsMap.get(configName);
            if (currentOverrideConfig) {
                const { overrideConfig: topLevelConfig, configName: originalConfigName } = currentOverrideConfig;
                overrideConfigs.push(...topLevelConfig);
                return { overrideConfigs, originalConfigName };
            }
        }
        return { overrideConfigs, originalConfigName: configName };
    }
    getTopologyForRunningEnvironments(runningEnvironments) {
        return Object.entries(runningEnvironments).reduce((acc, [envName, { port }]) => {
            acc[envName] = `http://localhost:${port}/${envName}`;
            return acc;
        }, {});
    }
    getRunningFeatures() {
        const runningFeatures = new Map();
        for (const [featureId, { runningEnvironments }] of this.runningFeatures) {
            const [featureName, configName] = featureId.split(path_1.delimiter);
            runningFeatures.set({ featureName, configName }, runningEnvironments);
        }
        return runningFeatures;
    }
    async closeEnvironment({ featureName, configName, }) {
        var _a;
        const featureId = `${featureName}${configName ? path_1.delimiter + configName : ''}`;
        const { runningEnvironments, com } = (_a = this.runningFeatures.get(featureId)) !== null && _a !== void 0 ? _a : {};
        if (!runningEnvironments || !com) {
            throw new Error(`there are no node environments running for ${featureName} and config ${configName}`);
        }
        com.dispose();
        for (const env of Object.values(runningEnvironments)) {
            await env.close();
        }
        this.runningFeatures.delete(featureId);
    }
    getFeaturesWithRunningEnvironments() {
        return Array.from(this.runningFeatures.keys()).map((runningFeature) => runningFeature.split(path_1.delimiter));
    }
    getTopology(featureName, configName) {
        var _a;
        const featureId = `${featureName}${configName ? path_1.delimiter + configName : ''}`;
        const { runningEnvironments } = (_a = this.runningFeatures.get(featureId)) !== null && _a !== void 0 ? _a : {};
        if (!runningEnvironments)
            return {};
        return this.getTopologyForRunningEnvironments(runningEnvironments);
    }
    async closeAll() {
        const allClosed = [];
        for (const runningEnvironmentForFeature of this.runningFeatures.values()) {
            runningEnvironmentForFeature.com.dispose();
            for (const env of Object.values(runningEnvironmentForFeature.runningEnvironments)) {
                allClosed.push(env.close());
            }
        }
        await Promise.all(allClosed);
        this.runningFeatures.clear();
    }
    async prepareEnvironment({ nodeEnv, featureName, bundlePath: outputPath, config, options, mode, com, features, }) {
        const { port, inspect } = this.options;
        const nodeEnvironmentOptions = {
            ...nodeEnv,
            bundlePath: outputPath,
            config,
            featureName,
            features: Array.from(features.entries()),
            options: Object.entries(options),
            inspect,
            context: this.context,
        };
        if (inspect || mode === 'forked') {
            if (inspect && mode !== 'forked') {
                console.warn(`Cannot inspect env without forking new process.
                    Launchihg environment ${nodeEnv.name} on remote process.`);
            }
            const { childProcess, port, start } = await this.runRemoteNodeEnvironment(nodeEnvironmentOptions);
            const ipcHost = new engine_core_node_1.IPCHost(childProcess);
            // change previous host registration
            com.clearEnvironment(nodeEnv.name);
            com.registerMessageHandler(ipcHost);
            com.registerEnv(nodeEnv.name, ipcHost);
            com.handleReady({ from: nodeEnv.name });
            return {
                port,
                start,
            };
        }
        if (mode === 'new-server') {
            return await this.runEnvironmentInNewServer(port, nodeEnvironmentOptions);
        }
        const { start } = (0, ws_environment_1.runWSEnvironment)(this.socketServer, nodeEnvironmentOptions);
        return {
            start,
            port,
        };
    }
    async runEnvironmentInNewServer(port, serverEnvironmentOptions) {
        const { httpServer, port: realPort } = await (0, create_listening_server_1.safeListeningHttpServer)(port);
        const socketServer = new socket_io_1.default.Server(httpServer, { cors: {}, ...this.socketServerOptions });
        return {
            start: async () => {
                const { close } = await (0, ws_environment_1.runWSEnvironment)(socketServer, serverEnvironmentOptions).start();
                const openSockets = new Set();
                const captureConnections = (socket) => {
                    openSockets.add(socket);
                    socket.once('close', () => {
                        openSockets.delete(socket);
                    });
                };
                httpServer.on('connection', captureConnections);
                return {
                    close: async () => {
                        httpServer.off('connection', captureConnections);
                        for (const socket of openSockets) {
                            socket.destroy();
                        }
                        await new Promise((res, rej) => socketServer.close((e) => {
                            if (e) {
                                rej(e);
                            }
                            else {
                                res();
                            }
                        }));
                        await close();
                    },
                };
            },
            port: realPort,
        };
    }
    async getConfig(configName) {
        var _a;
        const config = [];
        const { configurations } = this.options;
        if (configurations && configName) {
            const configDefinition = configurations.get(configName);
            if (!configDefinition) {
                const configNames = Array.from(configurations.keys());
                throw new Error(`cannot find config "${configName}". available configurations: ${configNames.join(', ')}`);
            }
            for (const definition of configDefinition) {
                try {
                    if (Array.isArray(definition)) {
                        config.push(...definition);
                    }
                    else {
                        config.push(...(await (_a = definition.filePath, Promise.resolve().then(() => __importStar(require(_a))))).default);
                    }
                }
                catch (e) {
                    console.error(e);
                }
            }
        }
        return config;
    }
    async runRemoteNodeEnvironment(options) {
        const { remoteNodeEnvironment, process: childProc } = await (0, remote_node_environment_1.startRemoteNodeEnvironment)(cliEntry, {
            inspect: this.options.inspect,
            port: this.options.port,
            socketServerOptions: this.socketServerOptions,
            requiredPaths: this.options.requiredPaths,
        });
        const port = await remoteNodeEnvironment.getRemotePort();
        return {
            start: async () => {
                const startMessage = new Promise((resolve) => {
                    remoteNodeEnvironment.subscribe((message) => {
                        if ((0, types_1.isEnvironmentStartMessage)(message)) {
                            resolve();
                        }
                    });
                });
                remoteNodeEnvironment.postMessage({
                    id: 'start',
                    envName: options.name,
                    data: options,
                });
                await startMessage;
                return {
                    close: async () => {
                        await new Promise((resolve) => {
                            const closeSubscriber = (message) => {
                                if (message.id === 'close') {
                                    remoteNodeEnvironment.unsubscribe(closeSubscriber);
                                    resolve();
                                }
                            };
                            remoteNodeEnvironment.subscribe(closeSubscriber);
                            remoteNodeEnvironment.postMessage({
                                id: 'close',
                                envName: options.name,
                            });
                        });
                        return remoteNodeEnvironment.dispose();
                    },
                };
            },
            port,
            childProcess: childProc,
        };
    }
}
exports.NodeEnvironmentsManager = NodeEnvironmentsManager;
//# sourceMappingURL=node-environments-manager.js.map