"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createIPC = void 0;
const engine_core_1 = require("@wixc3/engine-core");
const engine_core_node_1 = require("@wixc3/engine-core-node");
const cross_performance_1 = __importDefault(require("@wixc3/cross-performance"));
cross_performance_1.default.clearMeasures;
const ws_environment_1 = require("./ws-environment");
const types_1 = require("./types");
function createIPC(remoteProcess, socketServer, { port, onClose }) {
    const environments = {};
    const messageHandler = async (message) => {
        var _a, _b, _c, _d;
        if ((0, types_1.isEnvironmentPortMessage)(message)) {
            remoteProcess.postMessage({ id: 'port-request', payload: { port } });
        }
        else if ((0, types_1.isEnvironmentStartMessage)(message)) {
            // clearing because if running features one after the other on same engine, it is possible that some measuring were done on disposal of stuff, and the measures object will not be re-evaluated, so cleaning it
            cross_performance_1.default.clearMarks();
            cross_performance_1.default.clearMeasures();
            const ipcHost = new engine_core_node_1.IPCHost(process);
            // re-mapping all provided connected environments to use the IPC host, to allow, through the parent process, access other node environments
            const connectedEnvironments = {};
            //TODO: check
            for (const [envName, config] of (_a = message.data.config) !== null && _a !== void 0 ? _a : []) {
                if (envName === engine_core_1.COM.id) {
                    const typedConfig = (_b = config.config) !== null && _b !== void 0 ? _b : {};
                    const { connectedEnvironments: definedConnectedEnvironments } = typedConfig;
                    if (definedConnectedEnvironments) {
                        for (const [envToken, envRecord] of Object.entries(definedConnectedEnvironments)) {
                            if (!connectedEnvironments[envToken]) {
                                connectedEnvironments[envToken] = {
                                    ...envRecord,
                                    host: ipcHost,
                                };
                            }
                        }
                        typedConfig === null || typedConfig === void 0 ? true : delete typedConfig.connectedEnvironments;
                    }
                }
            }
            connectedEnvironments[engine_core_node_1.LOCAL_ENVIRONMENT_INITIALIZER_ENV_ID] = {
                host: ipcHost,
                id: engine_core_node_1.LOCAL_ENVIRONMENT_INITIALIZER_ENV_ID,
                registerMessageHandler: true,
            };
            const { close } = await (0, ws_environment_1.runWSEnvironment)(socketServer, {
                ...message.data,
                config: [
                    ...((_c = message.data.config) !== null && _c !== void 0 ? _c : []),
                    engine_core_1.COM.use({
                        config: {
                            connectedEnvironments,
                        },
                    }),
                ],
            }).start();
            environments[message.envName] = close;
            remoteProcess.postMessage({ id: 'start' });
        }
        else if ((0, types_1.isEnvironmentCloseMessage)(message) && environments[message.envName]) {
            await environments[message.envName]();
            remoteProcess.off('message', messageHandler);
            cross_performance_1.default.clearMarks();
            cross_performance_1.default.clearMeasures();
            if (onClose) {
                await onClose();
            }
            remoteProcess.postMessage({ id: 'close' });
            // clears all listeners. does not force close
            (_d = remoteProcess.terminate) === null || _d === void 0 ? void 0 : _d.call(remoteProcess);
        }
        else if ((0, types_1.isEnvironmentMetricsRequestMessage)(message)) {
            const metricsMessage = {
                id: 'metrics-response',
                payload: {
                    marks: cross_performance_1.default.getEntriesByType('mark'),
                    measures: cross_performance_1.default.getEntriesByType('measure'),
                },
            };
            remoteProcess.postMessage(metricsMessage);
        }
    };
    remoteProcess.on('message', messageHandler);
}
exports.createIPC = createIPC;
//# sourceMappingURL=process-communication.js.map