/**
 * Disposables allow adding of disposal async functions,
 * when dispose is called, these functions will be run sequentially
 */
export function createDisposables() {
    const disposables = new Set();
    return {
        async dispose() {
            const toDispose = Array.from(disposables).reverse();
            disposables.clear();
            for (const dispose of toDispose) {
                await dispose();
            }
        },
        add(disposable) {
            if (typeof disposable === 'function') {
                disposables.add(disposable);
            }
            else {
                disposables.add(() => disposable.dispose());
            }
        },
    };
}
//# sourceMappingURL=create-disposables.js.map