"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.watchClickOutside = void 0;
function watchClickOutside(topLayer, backdrop) {
    let pointerDownLayers = null;
    const onPointerDown = () => {
        // set available layers while mousedown to prevent any layer that
        // is created between mousedown and click to have it's click-outside called
        pointerDownLayers = topLayer.generateDisplayList();
    };
    const onClick = (event) => {
        const layers = pointerDownLayers || topLayer.generateDisplayList();
        pointerDownLayers = null;
        const target = event.target;
        const isBackdropClicked = target === backdrop.block;
        const backdropParentIndex = isBackdropClicked ? Number(backdrop.block.style.zIndex) - 1 : 0;
        // find clicked layer & layers that might require the event
        const layersToInform = new Set();
        let layerClickedIn = null;
        for (const layer of layers) {
            const element = layer.element;
            const layerIndex = element ? Number(element.style.zIndex) : -1;
            if ((element &&
                target.compareDocumentPosition(element) &
                    document.DOCUMENT_POSITION_CONTAINS) ||
                (isBackdropClicked && backdropParentIndex === layerIndex)) {
                layerClickedIn = layer;
            }
            else if (layer.settings.onClickOutside &&
                (!isBackdropClicked || backdropParentIndex < layerIndex)) {
                layersToInform.add(layer);
            }
        }
        // remove parents from inform set
        let current = layerClickedIn;
        while (current) {
            layersToInform.delete(current);
            current = current.parentLayer;
        }
        // inform click outside
        for (const { settings } of layersToInform) {
            settings.onClickOutside(target);
        }
    };
    window.addEventListener(`pointerdown`, onPointerDown, { capture: true });
    window.addEventListener(`click`, onClick, { capture: true });
    return {
        stop() {
            window.removeEventListener(`pointerdown`, onPointerDown);
            window.removeEventListener(`click`, onClick);
        },
    };
}
exports.watchClickOutside = watchClickOutside;
//# sourceMappingURL=click-outside.js.map