"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.popover = void 0;
const layout_overlay_1 = require("./layout-overlay");
const keep_in_view_1 = require("./keep-in-view");
const defaultOptions = {
    positionX: `center`,
    positionY: `after`,
    avoidAnchor: false,
    matchWidth: false,
    matchHeight: false,
};
const popover = () => {
    let bindPosition = null;
    let openedOverlay = null;
    let options = defaultOptions;
    const getOptions = () => ({
        x: options.positionX,
        y: options.positionY,
        width: options.matchWidth,
        height: options.matchHeight,
        onOverflow: (data) => (0, keep_in_view_1.keepInView)(data, { avoidAnchor: options.avoidAnchor }),
    });
    return {
        open({ anchor, overlay }, overlayOptions = {}) {
            if (bindPosition) {
                return;
            }
            openedOverlay = overlay;
            options = {
                ...defaultOptions,
                ...overlayOptions,
            };
            bindPosition = (0, layout_overlay_1.layoutOverlay)(anchor, overlay, getOptions());
        },
        close() {
            openedOverlay === null || openedOverlay === void 0 ? void 0 : openedOverlay.classList.add(layout_overlay_1.layoutOverlay.NOT_PLACED);
            bindPosition === null || bindPosition === void 0 ? void 0 : bindPosition.stop();
            bindPosition = null;
        },
        updateOptions(newOptions) {
            if (bindPosition) {
                options = {
                    ...options,
                    ...newOptions,
                };
                bindPosition.updateOptions(getOptions());
            }
        },
        isOpen() {
            return !!bindPosition;
        },
    };
};
exports.popover = popover;
//# sourceMappingURL=popover.js.map