"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createLayer = void 0;
const destroyLayer = Symbol(`destroy-layer`);
function createLayer({ parentLayer, onChange = noop, init, destroy, extendLayer, defaultSettings = {}, settings, } = { defaultSettings: {} }) {
    const children = [];
    const extendedData = extendLayer ? extendLayer : {};
    const layer = {
        ...extendedData,
        parentLayer: parentLayer || null,
        createLayer: (nestedOptions = {}) => {
            const childLayer = createLayer({
                parentLayer: layer,
                onChange: (reason, layer) => {
                    if (nestedOptions.onChange) {
                        nestedOptions.onChange();
                    }
                    onChange(reason, layer);
                },
                extendLayer,
                init,
                destroy,
                defaultSettings,
                settings: nestedOptions.settings,
            });
            children.push(childLayer);
            onChange(`create`, childLayer);
            return childLayer;
        },
        removeLayer: (layer) => {
            const layerIndex = children.indexOf(layer);
            if (layerIndex !== -1) {
                children.splice(layerIndex, 1);
                layer[destroyLayer]();
                onChange(`remove`, layer);
            }
        },
        [destroyLayer]: () => {
            if (destroy) {
                for (const childLayer of children) {
                    childLayer[destroyLayer]();
                }
                destroy(layer);
            }
        },
        generateDisplayList: () => {
            const list = [layer];
            for (const childLayer of children) {
                list.push(...childLayer.generateDisplayList());
            }
            return list;
        },
    };
    if (init) {
        init(layer, {
            ...defaultSettings,
            ...(settings || {}), // ToDo: fix fallback for undefined
        });
    }
    return layer;
}
exports.createLayer = createLayer;
const noop = () => {
    /*do nothing*/
};
//# sourceMappingURL=layer.js.map