"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createTempDirectorySync = exports.createTempDirectory = void 0;
const os_1 = require("os");
const path_1 = require("path");
const fs_1 = require("fs");
const rimraf_1 = __importDefault(require("rimraf"));
/**
 * Create an empty, unique directory in the current OS's temp directory.
 *
 * @param prefix optional prefix to add to the random name. (default: "temp-")
 * @returns an absolute `path` and a `remove()` function.
 */
async function createTempDirectory(prefix = 'temp-') {
    const path = await fs_1.promises.mkdtemp((0, path_1.join)((0, os_1.tmpdir)(), prefix));
    const remove = () => (0, rimraf_1.default)(path);
    return { path: await fs_1.promises.realpath(path), remove };
}
exports.createTempDirectory = createTempDirectory;
/**
 * Create an empty, unique directory in the current OS's temp directory.
 *
 * @param prefix optional prefix to add to the random name. (default: "temp-")
 * @returns an absolute `path` and a `remove()` function.
 */
function createTempDirectorySync(prefix = 'temp-') {
    const path = (0, fs_1.mkdtempSync)((0, path_1.join)((0, os_1.tmpdir)(), prefix));
    const remove = () => rimraf_1.default.sync(path);
    return { path: (0, fs_1.realpathSync)(path), remove };
}
exports.createTempDirectorySync = createTempDirectorySync;
//# sourceMappingURL=create-temp-directory.js.map