"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.scenarioPlugin = exports.expectWindowScroll = exports.expectElementScroll = exports.expectElementsStyle = exports.expectElementStyle = exports.expectElementText = exports.expectElements = exports.expectElement = exports.waitForElement = exports.writeAction = exports.clickAction = exports.hoverAction = exports.scrollAction = exports.maxScroll = exports.RenderWrapper = exports.ScenarioRenderer = void 0;
const board_core_1 = require("@wixc3/board-core");
const chai_1 = require("chai");
const promise_assist_1 = require("promise-assist");
const react_1 = __importStar(require("react"));
const test_utils_1 = __importDefault(require("react-dom/test-utils"));
const plugin_controls_1 = require("../plugin-controls/plugin-controls");
const scenario_plugin_st_css_1 = require("./scenario-plugin.st.css");
const ScenarioRenderer = (props) => {
    var _a;
    const { resetBoard, events: propEvents } = props;
    const [events, updateEvents] = (0, react_1.useState)(propEvents);
    const [btnText, updateButtonText] = (0, react_1.useState)((_a = propEvents[0]) === null || _a === void 0 ? void 0 : _a.title);
    const [autoRunning, triggerAutoRun] = (0, react_1.useState)(false);
    const [runningAction, setRunning] = (0, react_1.useState)(false);
    const highlight = (0, react_1.useMemo)(() => window.document.createElement('div'), []);
    const clearHighlight = (0, react_1.useCallback)(() => {
        highlight.removeAttribute('style');
        highlight.removeAttribute('class');
    }, [highlight]);
    const setHighlightedElement = (0, react_1.useCallback)((selector) => {
        if (!selector) {
            clearHighlight();
            return;
        }
        const target = actionTarget(selector);
        if (target && target instanceof Element) {
            const rect = target.getBoundingClientRect();
            highlight.setAttribute('style', `position: absolute; top: ${rect.top}px; left: ${rect.left}px; height:${rect.height}px; width:${rect.width}px;`);
            highlight.setAttribute('class', scenario_plugin_st_css_1.classes.item);
        }
    }, [clearHighlight, highlight]);
    const hoverAction = () => {
        const current = events[0];
        if (current === null || current === void 0 ? void 0 : current.highlightSelector) {
            setHighlightedElement(current.highlightSelector);
        }
        else {
            clearHighlight();
        }
    };
    const resetAll = (0, react_1.useCallback)(() => {
        resetBoard();
        updateButtonText(propEvents[0].title);
        updateEvents(propEvents);
    }, [propEvents, resetBoard]);
    const runAction = (0, react_1.useCallback)(() => {
        var _a;
        const current = events[0];
        if (current) {
            const onTaskFailed = (err) => {
                updateEvents(events.slice(1));
                if (err instanceof Error) {
                    updateButtonText('Error:' + err.message);
                }
                else {
                    updateButtonText('Error ');
                }
            };
            const onTaskDone = () => {
                const next = events[1];
                updateButtonText((next === null || next === void 0 ? void 0 : next.title) || 'Done!');
                if (!next) {
                    triggerAutoRun(false);
                }
                updateEvents(events.slice(1));
            };
            try {
                const res = current.execute();
                if (res) {
                    updateButtonText('task in progress');
                    return res.then(onTaskDone).catch(onTaskFailed);
                }
                else {
                    return onTaskDone();
                }
            }
            catch (err) {
                return onTaskFailed(err);
            }
        }
        else {
            resetAll();
        }
        setHighlightedElement((_a = events[1]) === null || _a === void 0 ? void 0 : _a.highlightSelector);
    }, [events, setHighlightedElement, resetAll]);
    const runActions = (0, react_1.useCallback)(() => {
        triggerAutoRun(true);
    }, []);
    (0, react_1.useEffect)(() => {
        const run = async () => {
            await (0, promise_assist_1.sleep)(props.slowMo || 10);
            if (autoRunning && !runningAction) {
                setRunning(true);
                await runAction();
                setRunning(false);
            }
        };
        void run();
    }, [autoRunning, props.slowMo, runAction, runningAction]);
    return (react_1.default.createElement("div", { className: scenario_plugin_st_css_1.classes.root },
        react_1.default.createElement("div", { className: (0, scenario_plugin_st_css_1.st)(scenario_plugin_st_css_1.classes.header, { skipped: props.skip }) },
            props.title || 'unnamedScenario',
            react_1.default.createElement("button", { className: scenario_plugin_st_css_1.classes.reset, onClick: resetAll, disabled: events.length === 0 }, "Restart"),
            react_1.default.createElement("button", { className: scenario_plugin_st_css_1.classes.reset, onClick: () => {
                    void runActions();
                } }, "Run")),
        react_1.default.createElement("div", { className: scenario_plugin_st_css_1.classes.content },
            react_1.default.createElement("div", null, events.length > 0 ? `${events.length} events left to run` : ''),
            react_1.default.createElement("div", { className: scenario_plugin_st_css_1.classes.controls },
                react_1.default.createElement("button", { onClick: () => {
                        void runAction();
                    }, onMouseLeave: clearHighlight, onMouseMove: hoverAction }, btnText)))));
};
exports.ScenarioRenderer = ScenarioRenderer;
const RenderWrapper = (props) => {
    const [boardKey, rerenderBoard] = (0, react_1.useReducer)((n) => n + 1, 0);
    const resetBoard = () => {
        var _a;
        rerenderBoard();
        (_a = props.resetBoard) === null || _a === void 0 ? void 0 : _a.call(props);
    };
    return (0, plugin_controls_1.renderInPluginControls)(react_1.default.createElement(react_1.default.Fragment, { key: boardKey }, props.board), react_1.default.createElement(exports.ScenarioRenderer, { skip: props.skip, title: props.title || 'untitled', events: props.events, resetBoard: resetBoard }), 'scenario ' + props.title);
};
exports.RenderWrapper = RenderWrapper;
const actionTarget = (selector) => {
    if (selector) {
        return window.document.querySelector(selector);
    }
    return window;
};
const maxScroll = (target, isVertical) => {
    if (target === window) {
        target = window.document.body;
    }
    const bounding = target.getBoundingClientRect();
    return Math.floor(isVertical
        ? target.scrollHeight - bounding.height
        : target.scrollWidth - bounding.width);
};
exports.maxScroll = maxScroll;
const scrollAction = (pos, isVertical = true, selector, timeout = 2000) => {
    return {
        title: 'Scroll ' + (selector || 'window') + ' to ' + (pos === -1 ? (isVertical ? 'bottom' : 'right-most') : pos),
        execute: async () => {
            const target = actionTarget(selector);
            const usedPos = target && pos === -1 ? (0, exports.maxScroll)(target, isVertical) : pos;
            if (target) {
                if (isVertical) {
                    target.scrollTo({
                        top: usedPos,
                        behavior: 'smooth',
                    });
                }
                else {
                    target.scrollTo({
                        left: usedPos,
                        behavior: 'smooth',
                    });
                }
            }
            return (0, promise_assist_1.waitFor)(() => {
                const currentPos = !target
                    ? 0
                    : target instanceof Window
                        ? isVertical
                            ? target.scrollY
                            : target.scrollX
                        : isVertical
                            ? target.scrollTop
                            : target.scrollLeft;
                (0, chai_1.expect)(Math.round(currentPos)).to.approximately(Math.round(usedPos), 2);
            }, { timeout });
        },
        timeout,
        highlightSelector: selector,
    };
};
exports.scrollAction = scrollAction;
const hoverAction = (selector, timeout = 2000) => {
    return {
        title: 'Hover ' + (selector || 'window'),
        execute: () => {
            const target = actionTarget(selector);
            if (target) {
                target.dispatchEvent(new MouseEvent('mousemove', {
                    bubbles: true,
                    relatedTarget: target,
                }));
            }
        },
        highlightSelector: selector,
        timeout,
    };
};
exports.hoverAction = hoverAction;
const clickAction = (selector, timeout = 2000) => {
    return {
        title: 'Click ' + (selector || 'window'),
        execute: () => {
            const target = actionTarget(selector);
            if (target) {
                target.dispatchEvent(new MouseEvent('mousedown', {
                    bubbles: true,
                    relatedTarget: target,
                }));
                target.dispatchEvent(new MouseEvent('click', {
                    bubbles: true,
                    relatedTarget: target,
                }));
                target.dispatchEvent(new MouseEvent('mouseup', {
                    bubbles: true,
                    relatedTarget: target,
                }));
            }
        },
        highlightSelector: selector,
        timeout,
    };
};
exports.clickAction = clickAction;
const writeAction = (selector, text, timeout = 2000) => {
    const title = `Write in "${text}" + ${selector}`;
    return {
        title,
        execute: async () => {
            const el = await (0, exports.waitForElement)(selector, title, timeout);
            if (el && el instanceof HTMLInputElement) {
                el.value = text;
                test_utils_1.default.Simulate.change(el, {
                    target: el,
                });
            }
        },
        highlightSelector: selector,
        timeout,
    };
};
exports.writeAction = writeAction;
const waitForElement = async (selector, title, timeout = 1000) => {
    await (0, promise_assist_1.waitFor)(() => {
        const el = window.document.querySelector(selector);
        if (!el) {
            throw new Error(title + ': element not found for selector ' + selector);
        }
    }, { timeout });
    return window.document.querySelector(selector);
};
exports.waitForElement = waitForElement;
const expectElement = (selector, expectation, title = 'expecting selector ' + selector, timeout = 2000) => {
    return {
        title,
        async execute() {
            const startTime = Date.now();
            const innerExec = async () => {
                const timePassed = Date.now() - startTime;
                const el = (await (0, exports.waitForElement)(selector, title, timeout - timePassed));
                if (expectation) {
                    try {
                        expectation(el);
                    }
                    catch (err) {
                        if (timePassed > timeout) {
                            throw err;
                        }
                        await (0, promise_assist_1.sleep)(Math.min(timeout - timePassed, 10));
                        return innerExec();
                    }
                }
            };
            return innerExec();
        },
        timeout,
        highlightSelector: selector,
    };
};
exports.expectElement = expectElement;
const expectElements = (selectors, expectation, title = 'expecting elements ' + selectors, timeout = 2000) => {
    return {
        title,
        timeout,
        async execute() {
            await (0, promise_assist_1.waitFor)(() => {
                const res = selectors.reduce((acc, selector) => {
                    const el = window.document.querySelector(selector);
                    if (!el) {
                        throw new Error(title + ': element not found for selector ' + selector);
                    }
                    acc[selector] = el;
                    return acc;
                }, {});
                if (expectation) {
                    expectation(res);
                }
            }, {
                timeout,
            });
        },
    };
};
exports.expectElements = expectElements;
const expectElementText = (selector, text, title = 'expecting text ' + text + ' for selector ' + selector, timeout = 2000) => {
    return {
        title,
        timeout,
        execute() {
            return (0, exports.expectElement)(selector, (el) => {
                if (!(el instanceof HTMLElement)) {
                    throw new Error(title + ': element at ' + selector + 'is not an HTMLElement');
                }
                (0, chai_1.expect)(el.innerText).to.equal(text);
            }, title, timeout).execute();
        },
        highlightSelector: selector,
    };
};
exports.expectElementText = expectElementText;
const expectElementStyle = (selector, expectedStyle, title = 'expectElementStyle ' + selector, timeout = 2000) => {
    const exp = (0, exports.expectElement)(selector, (el) => {
        const style = window.getComputedStyle(el);
        for (const [key, val] of Object.entries(expectedStyle)) {
            (0, chai_1.expect)(style[key]).to.eql(val);
        }
    }, title);
    return {
        title,
        execute() {
            return exp.execute();
        },
        highlightSelector: selector,
        timeout,
    };
};
exports.expectElementStyle = expectElementStyle;
const expectElementsStyle = (elements, title, timeout = 2000) => {
    return {
        title: title || 'expectElementsStyle ' + Object.keys(elements),
        execute() {
            for (const [selector, styles] of Object.entries(elements)) {
                (0, exports.expectElementStyle)(selector, styles, title, timeout);
            }
        },
        timeout,
    };
};
exports.expectElementsStyle = expectElementsStyle;
const expectElementScroll = (selector, expectedScroll, isHorizontal = false, title = 'expecting scroll ' + selector, timeout = 2000) => {
    return (0, exports.expectElement)(selector, (el) => {
        if (isHorizontal) {
            (0, chai_1.expect)(el.scrollLeft).to.eql(expectedScroll);
        }
        else {
            (0, chai_1.expect)(el.scrollTop).to.eql(expectedScroll);
        }
    }, title, timeout);
};
exports.expectElementScroll = expectElementScroll;
const expectWindowScroll = (expectedScroll, isHorizontal = false, title = 'expecting window scroll ') => {
    return {
        title,
        execute: () => {
            if (isHorizontal) {
                (0, chai_1.expect)(window.scrollX).to.eql(expectedScroll);
            }
            else {
                (0, chai_1.expect)(window.scrollY).to.eql(expectedScroll);
            }
        },
        timeout: 0,
    };
};
exports.expectWindowScroll = expectWindowScroll;
exports.scenarioPlugin = (0, board_core_1.createPlugin)()('scenario', {
    title: 'scenario',
    events: [],
    timeout: 2000,
    skip: false,
}, {
    wrapRender(props, _r, board) {
        return react_1.default.createElement(exports.RenderWrapper, { board: board, ...props });
    },
});
//# sourceMappingURL=scenario-plugin.js.map