"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useElementDimensions = exports.createSetableMutationObserver = exports.useSetableObserver = void 0;
const react_1 = require("react");
const common_1 = require("../common");
const hooks_1 = require("../hooks");
const getElementDimensions = (element) => {
    const rect = element === null || element === void 0 ? void 0 : element.getBoundingClientRect();
    return (0, common_1.getDimensionsFromRect)(rect);
};
const calculateDimensions = (ref, items, size, getId, measure, sizeCache, oldRes, setObserveTargets) => {
    const getDimensions = (item) => {
        if (size !== undefined) {
            return typeof size === 'function' ? size(item) : size;
        }
        return common_1.unmeasuredDimensions;
    };
    if (size !== undefined || !measure || !ref.current) {
        let changed = false;
        const res = items.reduce((acc, item) => {
            var _a, _b;
            const id = getId(item);
            const itemDimensions = getDimensions(item);
            acc[id] = itemDimensions;
            if (((_a = oldRes[id]) === null || _a === void 0 ? void 0 : _a.height) !== (itemDimensions === null || itemDimensions === void 0 ? void 0 : itemDimensions.height) || ((_b = oldRes[id]) === null || _b === void 0 ? void 0 : _b.width) !== (itemDimensions === null || itemDimensions === void 0 ? void 0 : itemDimensions.width)) {
                changed = true;
            }
            return acc;
        }, {});
        return {
            changed,
            res: changed ? res : oldRes,
        };
    }
    const elements = (0, common_1.childrenById)(ref.current);
    let changed = false;
    setObserveTargets === null || setObserveTargets === void 0 ? void 0 : setObserveTargets(Object.values(elements));
    const res = items.reduce((acc, item) => {
        var _a, _b, _c, _d;
        const id = getId(item);
        const element = elements[id];
        const cachedSize = sizeCache.get(id);
        if (cachedSize) {
            acc[id] = cachedSize;
        }
        else if (element) {
            const measured = getElementDimensions(element);
            acc[id] = measured;
            sizeCache.set(id, measured);
        }
        else {
            acc[id] = common_1.unmeasuredDimensions;
        }
        if (((_a = acc[id]) === null || _a === void 0 ? void 0 : _a.height) !== ((_b = oldRes[id]) === null || _b === void 0 ? void 0 : _b.height) || ((_c = acc[id]) === null || _c === void 0 ? void 0 : _c.width) !== ((_d = oldRes[id]) === null || _d === void 0 ? void 0 : _d.width)) {
            changed = true;
        }
        return acc;
    }, {});
    return {
        changed,
        res: changed ? res : oldRes,
    };
};
const useSetableObserver = (shouldMeasure) => {
    const listener = (0, react_1.useRef)(() => undefined);
    const observerRef = (0, react_1.useRef)();
    const observed = (0, react_1.useRef)(new Set());
    (0, react_1.useLayoutEffect)(() => {
        if (shouldMeasure) {
            observerRef.current = new ResizeObserver((ev, obs) => listener.current(ev, obs));
        }
        return () => {
            var _a;
            (_a = observerRef.current) === null || _a === void 0 ? void 0 : _a.disconnect();
        };
    }, [shouldMeasure]);
    const listen = (lis) => (listener.current = lis);
    const setTargets = (0, react_1.useCallback)((targets) => {
        const newObserved = new Set();
        const observer = observerRef.current;
        if (!observer) {
            return;
        }
        for (const target of targets) {
            newObserved.add(target);
            if (!observed.current.has(target)) {
                observer.observe(target);
            }
        }
        for (const oldTarget of [...observed.current]) {
            if (!newObserved.has(oldTarget)) {
                observer.unobserve(oldTarget);
            }
        }
        observed.current = newObserved;
    }, []);
    return {
        listen,
        setTargets,
    };
};
exports.useSetableObserver = useSetableObserver;
const createSetableMutationObserver = () => {
    let listener = () => undefined;
    const listen = (lis) => (listener = lis);
    const observer = new MutationObserver((ev, obs) => listener(ev, obs));
    return {
        listen,
        observer,
    };
};
exports.createSetableMutationObserver = createSetableMutationObserver;
const useElementDimensions = (ref, items, getId, getItemDimensions, observeSubtree = false) => {
    const shouldMeasure = getItemDimensions === false;
    const preMeasured = typeof getItemDimensions === 'boolean' ? undefined : getItemDimensions;
    const cache = (0, react_1.useRef)(new Map());
    const calculatedSize = (0, react_1.useMemo)(() => calculateDimensions(ref, items, preMeasured, getId, false, cache.current, {}).res, [getId, items, preMeasured, ref]);
    const unMeasuredDimensions = (0, react_1.useRef)(calculatedSize);
    const dimensions = (0, react_1.useRef)(calculatedSize);
    const updateDimensions = (0, react_1.useCallback)((newDimensions) => {
        for (const item in newDimensions) {
            dimensions.current[item] = newDimensions[item];
        }
    }, []);
    const delayedUpdateSizes = (0, hooks_1.useDelayedUpdateState)(updateDimensions);
    const { setTargets, listen } = (0, exports.useSetableObserver)(shouldMeasure);
    const { observer: mutationObserver, listen: mutationListener } = (0, react_1.useMemo)(exports.createSetableMutationObserver, []);
    const triggerDelayedUpdateSizes = (0, react_1.useCallback)(() => delayedUpdateSizes(() => {
        const { changed, res } = calculateDimensions(ref, items, preMeasured, getId, true, cache.current, dimensions.current, setTargets);
        if (!changed) {
            return hooks_1.unchanged;
        }
        return res;
    }), [ref, getId, items, delayedUpdateSizes, preMeasured, setTargets, dimensions]);
    mutationListener(() => {
        if (observeSubtree) {
            triggerDelayedUpdateSizes();
        }
    });
    listen((entries) => {
        for (const { target, contentRect } of entries) {
            const id = target.getAttribute('data-id');
            if (id) {
                cache.current.set(id, {
                    width: contentRect.width,
                    height: contentRect.height,
                });
            }
        }
        triggerDelayedUpdateSizes();
    });
    (0, react_1.useLayoutEffect)(() => {
        if (ref.current && observeSubtree) {
            mutationObserver.observe(ref.current, {
                childList: true,
                subtree: true,
            });
        }
        return () => {
            mutationObserver.disconnect();
        };
    }, [mutationObserver, observeSubtree, ref]);
    (0, react_1.useLayoutEffect)(() => {
        if (!shouldMeasure) {
            return;
        }
        const { changed, res } = calculateDimensions(ref, items, preMeasured, getId, true, cache.current, dimensions.current, setTargets);
        if (changed) {
            updateDimensions(res);
        }
    }, [ref, getId, shouldMeasure, items, getItemDimensions, preMeasured, setTargets, dimensions, updateDimensions]);
    if (!shouldMeasure) {
        unMeasuredDimensions.current = calculatedSize;
        return unMeasuredDimensions;
    }
    return dimensions;
};
exports.useElementDimensions = useElementDimensions;
//# sourceMappingURL=use-element-dimensions.js.map