"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.usePositionInParent = void 0;
const react_1 = require("react");
const common_1 = require("../common");
const use_delayed_update_1 = require("./use-delayed-update");
const getItemPositionInParent = (el) => {
    const box = el.getBoundingClientRect();
    const parentBox = el.parentElement.getBoundingClientRect();
    return {
        y: box.top - parentBox.top,
        x: box.left - parentBox.left,
    };
};
const isSamePosition = (p1, p2) => p1.x === p2.x && p1.y === p2.y;
const useAfterRenderEffect = (element, onElementUpdate, watch, def = null) => {
    const [state, update] = (0, react_1.useState)(def);
    const delayedUpdate = (0, use_delayed_update_1.useDelayedUpdateState)(update);
    (0, react_1.useEffect)(() => {
        if (watch === 'ignore') {
            return;
        }
        if (element.current) {
            delayedUpdate(() => onElementUpdate(element.current));
        }
    });
    return state;
};
const usePositionInParent = (element, watchPosition = false) => {
    const watch = typeof watchPosition === 'object' ? 'ignore' : watchPosition ? 'timer' : 'measure-once';
    const lastValRef = (0, react_1.useRef)();
    const onTimer = (0, react_1.useCallback)((el) => {
        const res = getItemPositionInParent(el);
        if (lastValRef.current && isSamePosition(res, lastValRef.current)) {
            return use_delayed_update_1.unchanged;
        }
        lastValRef.current = res;
        return res;
    }, []);
    return useAfterRenderEffect(element, onTimer, watch, common_1.unknownPosition);
};
exports.usePositionInParent = usePositionInParent;
//# sourceMappingURL=use-position.js.map