"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.useStateControls = void 0;
const react_1 = require("react");
const noop = () => undefined;
function useStateControls(options, 
//default value passed by the component, ignored if a value was passed in the options
defaultValue, useStateFunction = react_1.useState) {
    const status = options === undefined
        ? 'stateful'
        : Array.isArray(options)
            ? options[1] === true
                ? 'locked'
                : 'controlled'
            : 'stateful';
    const valueOrFactory = options === undefined ? defaultValue : Array.isArray(options) ? options[0] : options;
    const [value, setValue] = useStateFunction(valueOrFactory);
    return (0, react_1.useMemo)(() => {
        switch (status) {
            case 'controlled': {
                const [funcOrValue, setter] = options;
                if (typeof funcOrValue === 'function') {
                    return [funcOrValue(), setter];
                }
                return options;
            }
            case 'locked':
                return [value, noop];
            case 'stateful':
                return [value, setValue];
        }
    }, [status, options, value, setValue]);
}
exports.useStateControls = useStateControls;
//# sourceMappingURL=use-state-controls.js.map