"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.useScrollListScrollToSelected = void 0;
const react_1 = require("react");
const scroll_into_view_if_needed_1 = __importDefault(require("scroll-into-view-if-needed"));
const useScrollListScrollToSelected = ({ scrollToSelection, scrollWindow, scrollListRef, items, getId, selected, averageItemSize, itemsDimensions, mountedItems, isHorizontal, extraRenderSize, scrollWindowSize, }) => {
    const loadingTimeout = (0, react_1.useRef)(0);
    const timeout = (0, react_1.useRef)(0);
    const isScrollingToSelection = (0, react_1.useRef)(false);
    const selectedIndex = (0, react_1.useMemo)(() => items.findIndex((i) => getId(i) === selected), [items, getId, selected]);
    const calculateDistance = (0, react_1.useCallback)(({ itemIndex, direction }) => {
        var _a;
        let distance = 0;
        for (let index = itemIndex; index !== selectedIndex; direction === 'down' ? index++ : index--) {
            const item = items[index];
            const id = getId(item);
            const { height, width } = itemsDimensions.current[id] || {
                height: averageItemSize,
                width: averageItemSize,
            };
            const size = (_a = (isHorizontal ? width : height)) !== null && _a !== void 0 ? _a : averageItemSize;
            distance += size;
        }
        distance += scrollWindowSize * extraRenderSize;
        if (direction === 'down') {
            distance += scrollWindowSize;
        }
        return Math.floor((direction === 'down' ? 1 : -1) * distance);
    }, [averageItemSize, extraRenderSize, getId, isHorizontal, items, itemsDimensions, scrollWindowSize, selectedIndex]);
    const cleanUp = () => {
        isScrollingToSelection.current = false;
        loadingTimeout.current = 0;
        clearTimeout(timeout.current);
    };
    const scrollTo = (0, react_1.useCallback)((selectedIndex, repeated = false) => {
        var _a;
        if (!scrollListRef.current) {
            return;
        }
        clearTimeout(timeout.current);
        const scrollIntoView = (selected, position) => {
            var _a;
            const node = (_a = scrollListRef.current) === null || _a === void 0 ? void 0 : _a.querySelector(`[data-id='${getId(items[selected])}']`);
            if (node === null) {
                timeout.current = window.setTimeout(() => isScrollingToSelection.current && scrollTo(selected, true));
            }
            else {
                (0, scroll_into_view_if_needed_1.default)(node, {
                    scrollMode: 'if-needed',
                    block: position,
                    inline: position,
                    boundary: scrollWindow === null || scrollWindow === void 0 ? void 0 : scrollWindow.current,
                });
                cleanUp();
            }
        };
        const scrollTarget = (_a = scrollWindow === null || scrollWindow === void 0 ? void 0 : scrollWindow.current) !== null && _a !== void 0 ? _a : window;
        const mountedIndexes = [...mountedItems.current].map((id) => items.findIndex((i) => getId(i) === id));
        const firstIndex = Math.min(...mountedIndexes);
        const lastIndex = Math.max(...mountedIndexes);
        let position = 'nearest';
        if (selectedIndex < firstIndex) {
            position = 'start';
            scrollTarget.scrollBy({ top: calculateDistance({ itemIndex: firstIndex, direction: 'up' }) });
        }
        else if (lastIndex < selectedIndex) {
            position = 'end';
            scrollTarget.scrollBy({ top: calculateDistance({ itemIndex: lastIndex, direction: 'down' }) });
        }
        timeout.current = window.setTimeout(() => scrollIntoView(selectedIndex, repeated ? 'center' : position));
    }, [scrollListRef, scrollWindow, mountedItems, items, getId, calculateDistance]);
    (0, react_1.useEffect)(() => {
        if (scrollToSelection &&
            selectedIndex > -1 &&
            mountedItems.current.size > 0 &&
            !isScrollingToSelection.current) {
            isScrollingToSelection.current = true;
            scrollTo(selectedIndex);
        }
        return () => {
            cleanUp();
        };
    }, [scrollToSelection, mountedItems, scrollTo, selectedIndex]);
};
exports.useScrollListScrollToSelected = useScrollListScrollToSelected;
//# sourceMappingURL=use-scroll-list-scroll-to-selected.js.map